C     ========================================================
c     ========================================================

c           COMPETENT INTERPOLATOR FOR
c                 SEAFLOOR DATA

c                    Version S 13 Sep 2007 (Post GSMFC)

c                     Begun 18Sep2003 CJJ 

c                  INSTAAR, Univ Colorado, Boulder
c                         Chris Jenkins
c                   chris.jenkins@colorado.edu

c     ===================== S S S S S S ======================

c      

c     ========================================================




c     MAIN()
c     I/O UNITS:
c     1 - ETOPO2 DOS binary file from NOAA

c     2 - Direct Access binary Topography (ETOPO2)
c     3 - Succinct reformat of input data (***_###d.TXT)
c     4 - Main seafloor data input (***_ONE or ***_WWD.txt)

c     5 - Reserved for I/O
c     6 - Reserved for I/O

c     7 - AV/SpA ASCII Grid - Cell Variance (***_###v.ASC)
c     8 - AV/SpA ASCII Grid - Uncertainty (***_###u.ASC)
c     9 - AV/SpA ASCII Grid - Cell Data Count (***_###n.ASC)

c     10 - Succinct reformat of calibration holdback data (***_###c.TXT)

c     11 - AV/SpA ASCII Grid - Seafloor Avg Depth (***_ITPb.ASC)
c     12 - AV/SpA ASCII Grid - Seafloor Slope (Sobel) (***_ITPs.ASC)
c     13 - AV/SpA ASCII Grid - Seafloor Roughness (SD) (***_ITPr.ASC)

c     14 - Direct access matrix of data count in each cell ('_datMTRX.tmp')
c     15 - pointers from one site to next in a cell ('_iXlink.tmp')

c     16 - Audit Report of the Process and Setup ('***_ITPa.TXT')

c     17 - Point file of VAL outputs: enables faster monitoring of results

c     18 - AV/SpA ASCII Grid - Interpolated Value (***_ITP_.ASC)
c     19 - DirectAccess data supplying file ('_iXdata.tmp')
c     20 - 
c     21 - 
c     22 - SETUP file (_***_ITP.set)
c     23 - 
c     24 - Cellwise spatial indexing ('iXgrid.tmp')


c     SUBROUTINES:
c     ascGRIDhdrs 
c       
c     prepareDATA(iPARAM)
c       
c     prepareBINS(iPARAM)
c       
c     gridAVG
c       
c     gridMED
c       
c     gridIDW(idwPWR)
c       
c     CALC3x3
c       
c     getTOPO(rTOPO)
c       
c     getHYPO(rLAT,rLON,rHYPO,sdHYPO,slHYPO)
c       
c     constructDA(iRECLbin)
c       
c     breakUpLINE(vS,lineS)
c       
c     sortN(N0,RA)
c       
c     coastalCLASSN
c       
c     setSRCH(iCx,iCy,kmSRCHrad)
c       
c     findGRIDcell(iCx,iCy,cellORsrchS,nnSRCH,wghtngTYPs,weightYNS)
c       

c     rMAXMIN(mxmnS,a,b,rMXMNnul)
c       
c     str2num(sNUM)
c       
c     num2str(rNUM)
c       
c     LTRIM(strS)
c       
c     midS(strS,startN,stopN)
c       
c     ran0(idum)
c       

      INTEGER nCOLS,nROWS
      INTEGER*4 iRECLbin
      INTEGER*4 xyREC

      REAL kmCELLsz
      REAL dgCELLsz,kmSRCHrad,kmSRCHradMX
      REAL latMN,lonMN,latMX,lonMX,
     &     BOXwdMN,BOXwdMX,
     &     BOXtop,BOXbot
      REAL SRCHlatMN,SRCHlatMX,SRCHlonMN,SRCHlonMX,SRCHtop,SRCHbot
      REAL pntW(32),pntY(32),pntX(32),pntZ(32),pntQ(32),pntV(32),
     &     pntELNav(32),pntELNsd(32),pntELNsl(32)
      REAL latDATmax,latDATmin,lonDATmax,lonDATmin
      REAL pi,d2r
      REAL latCOS
      REAL rngPARAM(32),measUNC,loclUNC

      CHARACTER*1 c00,c34,c39
      CHARACTER workDIRS*25,projectS*3
      CHARACTER interpTYPs*3
      CHARACTER*2048 linei5S,lineS
      CHARACTER preRUNbathyS*1,PREPAREdataS*1,calibrateS*1
      CHARACTER*3 typSRCHrad,adjstSRCHrad,wghtngTYPs
      CHARACTER*3 nbrTYPs,nnQUADs
      CHARACTER*1 metaDS, dumS
      CHARACTER cellORsrchS*4,weightYNS*1
c     CHARACTER*1 ynC
      CHARACTER*12 setfileS
      CHARACTER nmSUFFX1s*36,nmSUFFX2s*40,nmSUFFXs*76
      CHARACTER fnmSUFFXs*3
C     And for DATE_AND_TIME Aug 2007
      CHARACTER YMDs*8, HMSs*10, Zs*5
      INTEGER DT(8)

      COMMON /BOX/lonMN,latMN,BOXtop,boxBOT,nCOLS,nROWS
      COMMON /ELN/gdELN,gdSLP,gdSDV,gdMAX
      COMMON /CELL/cellLAT,cellLON
      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /GRD/iPn,inCELL,inSRCH
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /OUT/cellVAL,cellVAR,cellUNC,cellNUM
      COMMON /PNT/pntW,pntY,pntX,pntZ,pntQ,pntV,
     &        pntELNav,pntELNsd,pntELNsl,metaDS
      COMMON /PROJ/projectS,workDIRS
      COMMON /RADS/dgCELLsz,kmSRCHradMX,adjstSRCHrad,typSRCHrad
      COMMON /SRCH/SRCHlatMN,SRCHlatMX,SRCHlonMN,SRCHlonMX,
     &      SRCHtop,SRCHbot
      COMMON /CALIB/calibrateS
      COMMON /DATMXMN/latDATmin,lonDATmin,latDATmax,lonDATmax
      COMMON /SUFFIX/fnmSUFFXs

C     Constants ---------------------------
      c34=CHAR(34)
      c39=CHAR(39)
      c00=CHAR(0)
      pi=3.14152
      d2r=1./57.2
      lineS=c00
      rNUL=-99.00
      rnul9999=9999.
      rnul00=00.00
      iPn=0
      workDIRS='c:\dbSEABED\_db9\_Interp\'
C     -------------------------------------

c     ===================================================
c     Purpose of program
c     A. Calculate the site attribute & positional errors
c     B. Do an interpolation with uncertainties

c        CEL Uses only points within each cell (averages)
c        AVG Averages points sought by DiSTance or NAT neighbour
c        IDW Inverse Distance Weighted on points found by DST or NAT

c     Banner -----------------------
      WRITE(*,*) ''
      WRITE(*,*) '==============================================='
      WRITE(*,*) '        SEABED COMPETENT INTERPOLATOR'
      WRITE(*,*) ''
      WRITE(*,*) ' Program db9_ITPs.for ',
     &            'to interpolate seabed properties'
      WRITE(*,*) ''
      WRITE(*,*) '   INSTAAR CU Boulder CO ',
     &           'CJ Jenkins September 2007'
      WRITE(*,*) '==============================================='
      WRITE(*,*) ''
      WRITE(*,*) ''
c     ------------------------------

c     Get project and parameter (ie choose set file)-------------
1112  WRITE(*,*) 'Which project (3-letter code; eg. ADR,NYB)...'
      READ(*,'(a3)') projectS
      WRITE(*,*) ''
      IF ((projectS).EQ.'   ') THEN
         WRITE(*,*) 'Some problem with that - please retry'
         WRITE(*,*) ''
         GOTO 1112
      ENDIF
      setfileS='_'//projectS//'_ITP.SET'

1113  WRITE(*,*) 'The parameters are:...'
      WRITE(*,*) '   {12 Gravel; 13 Sand; 14 Mud; 15 Clay;',
     &           ' 16 Grainsize; 17 Sorting;'
      WRITE(*,*) '    19 ClsMshp; 21 RockMshp; 22 WeedMshp;',
     &           ' 23 Carbonate; 25 OrgCarbon;'
      WRITE(*,*) '    26 LgShearStr; 27 Porosity; 28 PWaveVel;',
     &           ' 30 LgCritShrStrs }'
      WRITE(*,*) 'Of these, which parameter number ? (i2)...'
      READ(*,'(i2)') iPARAM
      WRITE(*,*) ''
      IF ((iPARAM<12).OR.(iPARAM>30)) THEN
         WRITE(*,*) 'Some problem with that - please retry'
         GOTO 1113
      ELSE
c       Assign file name suffix
c       12-20 then 21-30
        nmSUFFX1s='GVL.SND.MUD.CLY.GRZ.SRT.   .CLM.   .'
        nmSUFFX2s='RCK.VEG.CRB.   .OCB.SHS.POR.PWV.   .CSS.'
        nmSUFFXs=nmSUFFX1s//nmSUFFX2s
        WRITE(*,*) 'nmSUFFXs:'
        WRITE(*,*) nmSUFFXs
        WRITE(*,*)
        fnmSUFFXs=nmSUFFXs((iPARAM-12)*4+1:(iPARAM-12+1)*4-1)
      ENDIF

      WRITE(*,*) 'Project SET file:  ',setfileS
      WRITE(*,*) '  and parameter to interpolate:  ',iPARAM
      WRITE(*,*) ''
c     CALL SLEEP(3)
c     -----------------------------------------------------------

c     Audit Report -----------------------------------------
      OPEN(16,FILE=workDIRS//'_'//projectS//'_'//fnmSUFFXs//'a.txt')
      WRITE(16,*)'Audit Report for Interpolation of dbSEABED data'
      WRITE(16,*)'***********************************************'

      WRITE(16,*) 'Project SET file:  ',setfileS
      WRITE(16,*) '  and parameter to interpolate:  ',iPARAM
      WRITE(16,*) 

c     SET INPUTS========================================
      OPEN(22,FILE=workDIRS//'_'//projectS//'_ITP.SET')

c     Read input SETUP file
c     (CJJ 2006: We will need to do some tests on these inputs)

      DO iI=1,100
c        Some lines alert to data in line following
         READ(22,'(a1024)',END=119) linei5S
c        WRITE(*,*) iI,'\',linei5S(1:32),'\'
c        Echo all lines to audit file:
c        WRITE(16,*) linei5S(1:LEN_TRIM(linei5S))

         IF ((linei5S(1:1).EQ.c00).OR.
     &      (linei5S(1:1).EQ.c39)) THEN
c           It is a comment - do nothing
cc       ELSEIF (INDEX(linei5S,'=workDIRs')>0) THEN
cc          workDIRs=linei5S(1:3)
cc          WRITE(*,*) 'workDIRs=',workDIRs
         ELSEIF (INDEX(linei5S,'=preRUNbathyS')>0) THEN
            preRUNbathyS=linei5S(1:1)
c           WRITE(*,*) 'preRUNbathyS=',preRUNbathyS
         ELSEIF (INDEX(linei5S,'=calibrateS')>0) THEN
            calibrateS=linei5S(1:1)
c           WRITE(*,*) 'calibrateS=',calibrateS
         ELSEIF (INDEX(linei5S,'=PREPAREdataS')>0) THEN
            PREPAREdataS=linei5S(1:1)
c           WRITE(*,*) 'PREPAREdataS=',PREPAREdataS

         ELSEIF (INDEX(linei5S,'=interpTYPs')>0) THEN
            interpTYPs=linei5S(1:3)
c           WRITE(*,*) 'interpTYPs=',interpTYPs
         ELSEIF (INDEX(linei5S,'=idwPWR')>0) THEN
            idwPWR=INT(str2num(linei5S(1:3)//'.                '))
c           WRITE(*,*) 'idwPWR=',idwPWR
         ELSEIF (INDEX(linei5S,'=iPARAM')>0) THEN
            iPARAM=INT(str2num(linei5S(1:2)//'.                '))
c           WRITE(*,*) 'iPARAM=',iPARAM
         ELSEIF (INDEX(linei5S,'=wghtngTYPs')>0) THEN
            wghtngTYPs=linei5S(1:3)
c           WRITE(*,*) 'wghtngTYPs=',wghtngTYPs

         ELSEIF (INDEX(linei5S,'=nbrTYPs')>0) THEN
            nbrTYPs=linei5S(1:3)
            WRITE(*,*) 'nbrTYPs=',nbrTYPs
         ELSEIF (INDEX(linei5S,'=nnQUADs')>0) THEN
            nnQUADs=linei5S(1:3)
c           WRITE(*,*) 'nnQUADs=',nnQUADs
         ELSEIF (INDEX(linei5S,'=nnSRCH')>0) THEN
            nnSRCH=INT(str2num(linei5S(1:3)//'.                '))
c           WRITE(*,*) 'nnSRCH=',nnSRCH
         ELSEIF (INDEX(linei5S,'=typSRCHrad')>0) THEN
            typSRCHrad=linei5S(1:3)
c           WRITE(*,*) 'typSRCHrad=',typSRCHrad
         ELSEIF (INDEX(linei5S,'=kmSRCHradMX')>0) THEN
            kmSRCHradMX=INT(str2num(linei5S(1:3)//'.                '))
c           WRITE(*,*) 'kmSRCHradMX=',kmSRCHradMX
         ELSEIF (INDEX(linei5S,'=adjstSRCHrad')>0) THEN
            adjstSRCHrad=linei5S(1:3)
c           WRITE(*,*) 'adjstSRCHrad=',adjstSRCHrad

         ELSEIF (INDEX(linei5S,'=dgCELLsz')>0) THEN
            dgCELLsz=str2num(linei5S(1:8)//'                ')
c           WRITE(*,*) 'dgCELLsz=',dgCELLsz
         ELSEIF (INDEX(linei5S,'=latMN')>0) THEN
            latMN=str2num(linei5S(1:10)//'                ')
c           WRITE(*,*) 'latMN=',latMN
         ELSEIF (INDEX(linei5S,'=lonMN')>0) THEN
            lonMN=str2num(linei5S(1:10)//'                ')
c           WRITE(*,*) 'lonMN=',lonMN
         ELSEIF (INDEX(linei5S,'=nCOLS')>0) THEN
            nCOLS=NINT(str2num(linei5S(1:4)//'                '))
c           WRITE(*,*) 'nCOLS=',nCOLS
         ELSEIF (INDEX(linei5S,'=nROWS')>0) THEN
            nROWS=NINT(str2num(linei5S(1:4)//'                '))
c           WRITE(*,*) 'nROWS=',nROWS
         ELSEIF (INDEX(linei5S,'=BOXwdMN')>0) THEN
            iBOXwdMN=INT(str2num(linei5S(1:5)//'.                '))
            BOXwdMN=REAL(iBOXwdMN)
c           WRITE(*,*) 'BOXwdMN=',BOXwdMN
         ELSEIF (INDEX(linei5S,'=BOXwdMX')>0) THEN
            iBOXwdMX=INT(str2num(linei5S(1:5)//'.                '))
            BOXwdMX=REAL(iBOXwdMX)
c           WRITE(*,*) 'BOXwdMX=',BOXwdMX
         ELSEIF (INDEX(linei5S,'=BOXtop')>0) THEN
            BOXtop=str2num(linei5S(1:6)//'                ')
c           WRITE(*,*) 'BOXtop=',BOXtop
         ELSEIF (INDEX(linei5S,'=BOXbot')>0) THEN
            BOXbot=str2num(linei5S(1:6)//'                ')
c           WRITE(*,*) 'BOXbot=',BOXbot
         ELSEIF (INDEX(linei5S,'=EndSetup')>0) THEN
            WRITE(*,*) 'ENDSETUP'
            GOTO 119
         ENDIF
      ENDDO
119   CLOSE(22)

      WRITE(*,*) ''
      WRITE(*,*) 'Echo the inputs to screen ======================'
      WRITE(*,*) 'workDIRS          { ',workDIRS,' }'
      WRITE(*,*) 'projectS          { ',projectS,' }'
      WRITE(*,*) 'preRUNbathyS      { ',preRUNbathyS,' }'
      WRITE(*,*) 'PREPAREdataS / calibrateS { ',
     &            PREPAREdataS,' / ',calibrateS,' }'
      WRITE(*,*) 'iPARAM            { ',iPARAM,' }'
      WRITE(*,*) 'wghtngTYPs        { ',wghtngTYPs,' }'
      WRITE(*,*) 'interpTYPs        { ',interpTYPs,' }'
      WRITE(*,*) 'nbrTYPs / nnQUADs { ',nbrTYPs,':',nnQUADs,' }'
      WRITE(*,*) 'nnSRCH            { ',nnSRCH,' }'
      WRITE(*,*) 'typSRCHrad        { ',typSRCHrad,' }'
      WRITE(*,*) 'adjstSRCHrad      { ',adjstSRCHrad,' }'
      WRITE(*,*) 'kmSRCHradMX       { ',kmSRCHradMX,' }'
      WRITE(*,*) 'dgCELLsz          { ',dgCELLsz,' }'
      WRITE(*,*) 'nROWS,nCOLS       { ',nROWS,nCOLS,' }'
      WRITE(*,*) 'latMN,lonMN       { ',latMN,lonMN,' }'
      WRITE(*,*) 'iBOXwdMN,iBOXwdMX { ',iBOXwdMN,iBOXwdMX,' }'
      WRITE(*,*) 'BOXtop,BOXbot     { ',BOXtop,BOXbot,' }'
      WRITE(*,*) ''

c     And for the audit report ....
      WRITE(16,*)
      WRITE(*,*)  'Echo the inputs ================================'
      WRITE(16,*) 'workDIRS          { ',workDIRS,' }'
      WRITE(16,*) 'projectS          { ',projectS,' }'
      WRITE(16,*) 'preRUNbathyS      { ',preRUNbathyS,' }'
      WRITE(16,*) 'PREPAREdataS / calibrateS { ',
     &             PREPAREdataS,' / ',calibrateS,' }'
      WRITE(16,*) 'iPARAM            { ',iPARAM,' }'
      WRITE(16,*) 'wghtngTYPs        { ',wghtngTYPs,' }'
      WRITE(16,*) 'interpTYPs        { ',interpTYPs,' }'
      WRITE(16,*) 'nbrTYPs / nnQUADs { ',nbrTYPs,':',nnQUADs,' }'
      WRITE(16,*) 'nnSRCH            { ',nnSRCH,' }'
      WRITE(16,*) 'typSRCHrad        { ',typSRCHrad,' }'
      WRITE(16,*) 'adjstSRCHrad      { ',adjstSRCHrad,' }'
      WRITE(16,*) 'kmSRCHradMX       { ',kmSRCHradMX,' }'
      WRITE(16,*) 'dgCELLsz          { ',dgCELLsz,' }'
      WRITE(16,*) 'nROWS,nCOLS       { ',nROWS,nCOLS,' }'
      WRITE(16,*) 'latMN,lonMN       { ',latMN,lonMN,' }'
      WRITE(16,*) 'iBOXwdMN,iBOXwdMX { ',iBOXwdMN,iBOXwdMX,' }'
      WRITE(16,*) 'BOXtop,BOXbot     { ',BOXtop,BOXbot,' }'
      WRITE(16,*) '---------------------------------------------'
      WRITE(16,*)

      WRITE(*,*) 'End of Setup Inputs'
      WRITE(*,*) ''
      CALL SLEEP(3)
c     --------------------------------------------------<<<

c     PRELIMINARIES=======================================

C     August 2007:
      CALL DATE_AND_TIME(YMDs,HMSs,Zs,DT)
      iYR=DT(1)
      iMO=DT(2)
      iDY=DT(3)
      iHR=DT(5)
      iMN=DT(6)
c     CALL GETTIM(iHR,iMN,iSC,i100)
c     CALL GETDAT(iYR,iMO,iDY)
      WRITE(16,*) 
      WRITE(16,*)'************************************'
      WRITE(16,*) 'Start Time (HH MM) : Date(YY MM DD)=',
     &                   iHR,iMN,':',iDY,iMO,iYR
      WRITE(16,*)'************************************'
      WRITE(16,*) 
c     ------------------------------------------------------

c     -------------------------------------------------
      latMX=latMN+nROWS*dgCELLsz
      lonMX=lonMN+nCOLS*dgCELLsz

      WRITE(*,*) 'Map box:    ',latMX
      WRITE(*,*) lonMN,'        ',lonMX
      WRITE(*,*) '        ',latMN,'         ',
     &           'BOX(sideX/Y)',nCOLS,nROWS
      WRITE(*,*) ''
      IF ((latMX <= latMN).OR.(lonMX <= lonMN)) THEN
         WRITE(*,*) 'Lat/lon box is confused (stop)'
         CALL SLEEP(3)
         STOP
      ENDIF
c     -------------------------------------------------

c     Enter Search DIAM (but work with RADIUS) --------
      cosLATsrch=abs(COS((latMN+latMX)/2.*d2r))
      dgSRCHradMX=kmSRCHradMX/(60.*1.832*cosLATsrch)
c     Allow for COS(rLON)
      kmCELLsz=dgCELLsz*(60.*1.832*cosLATsrch)
c     -------------------------------------------------

c     Define search area ----------------------
c     Extend box by search radius all sides
      sRCHlatMN=latMN-dgSRCHradMX
      sRCHlatMX=latMX+dgSRCHradMX
      sRCHlonMN=lonMN-dgSRCHradMX
      sRCHlonMX=lonMX+dgSRCHradMX
c     Search WD bracket not implemented CJJ Oct 04

      WRITE(*,*) 'Search Box:',SRCHlatMX
      WRITE(*,*)SRCHlonMN,'         ',SRCHlonMX
      WRITE(*,*) '        ',SRCHlatMN
      WRITE(*,*) ''
c        (SRCH grid size)
c     -----------------------------------------

c     Check if a cell is completely covered by search -----------
      IF (typSRCHrad.EQ.'SQR') THEN
         IF (kmSRCHradMX*2.0<kmCELLsz) THEN
            WRITE(*,*) 'Search SQUARE sizes<Cell sizes (',
     &                   kmCELLsz,kmSRCHradMX
            WRITE(*,*) '... Proceed?'
            CALL SLEEP(3)
         ENDIF
      ELSEIF (typSRCHrad.EQ.'CIR') THEN
         IF (kmSRCHradMX*2.0<kmCELLsz * SQRT(2.0)) THEN
            WRITE(*,*) 'Search CIRCLE sizes<Cell sizes (',
     &                   kmCELLsz,kmSRCHradMX,')'
            WRITE(*,*) '... Proceed?'
            CALL SLEEP(3)
         ENDIF
      ELSE
         WRITE(*,*) 'Bad option with typSRCHrad (check inputs)'
         CALL SLEEP(3)
         STOP
      ENDIF 
c     -----------------------------------------------------------

c     Construct the direct access binary table -----------------
      iRECLbin=10800*6
c     WRITE(*,*) iRECLbin
c     CALL constructDA(iRECLbin)

c     Open the Direct Access file ready for obtaining values
      OPEN(2,FILE='C:\dbSEABED\_db9\_Bathy\_Etopo2\ETOPO2.dab',
     &        ACCESS='DIRECT',RECL=iRECLbin,FORM='UNFORMATTED',ERR=300)
c     OPEN(2,FILE='C:\dbSEABED\_db9\_Bathy\_Etopo2\ETOPO2.bin',
c    &        ACCESS='DIRECT',RECL=2,FORM='UNFORMATTED',ERR=300)
c     This will be used to associate an ELN & SLP to the cells
c       (the seafloor data already have WD's attached - see *_WWD file)
c           TEST IT:
c           CALL getTOPO(49.342,-120.675,rTOPO)
c           WRITE(*,*) 'Test Direct Access File: ',rLAT,rLON,rTOPO
      GOTO 301
300      WRITE(*,*) 'Input ETOPO2 DirAccess file not found',
     &                ' or empty on OPEN'
         CALL SLEEP(3)
         STOP
301   CONTINUE
c     -----------------------------------------

c     Make a coastal classification --------
c        and append that data to *_WWD data
c        ynC=c00
c        WRITE(*,*) 'Skip doing the Coastal Classification (.y/n) ?'
c        WRITE(*,*) '   (ie: go straight to sample Data Preparation)'
c        READ(*,'(a1)') ynC
c        IF ((ynC.EQ.'n').OR.(ynC.EQ.c00)) THEN
c           CALL coastalCLASSN
c        ELSE
c           Do nothing
c        ENDIF
c     --------------------------------------

c     Get the data organized first ----------------------------
      IF (PREPAREdataS.EQ.'y') THEN
         WRITE(*,*) ''

c        WRITE(*,*) '>>prepareDATA'
         CALL prepareDATA(iPARAM)

      ELSEIF (PREPAREdataS.EQ.'n') THEN
         WRITE(*,*) 'Using pre-prepared input data: OK?'
         WRITE(*,*) '     ***_###d.txt'
         WRITE(*,*) '     ***_###c.txt'
         WRITE(*,*) '     _DATMTRX.TMP'
         WRITE(*,*) '     _iXlink.tmp'
         WRITE(*,*) '     _iXdata.TMP'
         WRITE(*,*) ''
         CALL SLEEP(3)
      ELSE
         WRITE(*,*) 'Bad input at PREPAREdataS (stop):',PREPAREdataS
         CALL SLEEP(3)
         STOP
      ENDIF
c     ----------------------------------------------------------

c     Make the grid headers -------------------------
      CALL ascGRIDhdrs
c     This opens several ASC grid files
c     -----------------------------------------------

c     WRITE(*,*) 'Go on to interpolation ? (y/n)'
c     READ(*,'(a1)') dumS
c     WRITE(*,*) 'OK'
c     =================================================== End Setup

c     INTERPOLATOR====================================================
      WRITE(*,*) ''
      WRITE(*,*) 'Start interpolating '
      WRITE(*,*) ''

c     Open the direct access num data-by-cell counts ------------------
c     (99825 lines adriatic)
      OPEN(14,FILE='C:\dbSEABED\_db9\_Interp\_datMTRX.tmp',
     &        ACCESS='DIRECT',RECL=6,FORM='FORMATTED',ERR=304)
      GOTO 3040
304      WRITE(*,*) 'Error in direct access data counting: (stop)'
         CALL SLEEP(3)
         STOP
3040  CONTINUE
c     -----------------------------------------------------------------

c     Need the files that hold the spatially indexed data

c     The file that contains the data -----------------
c        (direct access via spatial indexes)
c     Pointwise
      OPEN(19,FILE='C:\dbSEABED\_db9\_Interp\_iXdata.tmp',
     &   ACCESS='DIRECT',RECL=98,FORM='FORMATTED')

c     A file for storing grid spatial indexes ----------
      OPEN(24,FILE='C:\dbSEABED\_db9\_Interp\_iXgrid.tmp',
     &   ACCESS='DIRECT',RECL=9,FORM='FORMATTED')

c     A file for storing links between data in each cell  ----
c     This will write 28char data for later Direct Access
      OPEN(15,FILE='C:\dbSEABED\_db9\_Interp\_iXlink.tmp',
     &   ACCESS='DIRECT',RECL=9,FORM='FORMATTED')


c     Will also need a land mask (prx=0) --------------
c     The working proximity file is ASCII direct access 
      OPEN(1,FILE=workDIRS//'_'//projectS//'_prx.tmp',
     &       ACCESS='DIRECT',RECL=6,FORM='FORMATTED')
c     -------------------------------------------------

c     Now interpolate for each cell working W-E and N-S --------------
      DO iCy=nROWS,1,-1

c        We will quicken by trimming grid Y scan: ---
c            to the point data extent (+-kmSRCHrad)
c        Latitude of grid cell
         cellLAT=latMN+dgCELLsz*(REAL(iCy-1)+.5)
c        WRITE(*,*) cellLAT,latDATmax,latDATmin

         IF ((cellLAT > latDATmax+dgSRCHradMX).OR.
     &      (cellLAT < latDATmin-dgSRCHradMX)) THEN
c           (Dump whole row of ASCII grid entry)
            DO iCxB=1,nCOLS
               WRITE(18,'(f6.2,a1)') rNUL,' '
c*             WRITE(7,'(f6.2,a1)') rNUL,' '
               WRITE(8,'(f6.2,a1)') rNUL,' '
c*             WRITE(9,'(f6.2,a1)') rnul00,' '

c              Longitude of grid cell (latitude already available)
               cellLON=lonMN+dgCELLsz*(REAL(iCxB-1)+.5)

c              Write the bathymetric context
c              CALL getTOPO(gdELN)
c              WRITE(*,*) 'Into calc3x3 A',d2r
               CALL calc3x3
c              Returns gdELN/SLP/SDV

c              WRITE(11,'(f6.0,a1)') gdELN,' '
c*             WRITE(12,'(f6.3,a1)') gdSLP,' '
c*             WRITE(13,'(f6.1,a1)') gdSDV,' '

            ENDDO
c           WRITE(*,*) 'Skip row (no near data)(iCy=):',iCy
            GOTO 890
c           (End of row)
         ENDIF
c           --------------------------------------------

             DO iCx=1,nCOLS

c               Longitude of grid cell
                cellLON=lonMN+dgCELLsz*(REAL(iCx-1)+.5)

c               Write the bathymetric context
c               CALL getTOPO(gdELN)
c                  WRITE(*,*) 'Into calc3x3 B'
                    CALL calc3x3
c                  Returns gdELN/SLP/SDV
c               WRITE(*,*) 'gdELN,gdSLP,gdSDV',gdELN,gdSLP,gdSDV
c               ----------------------------------------------------

c               Shortcut if cell is on land (by Proximity)------
c                (in effect a masking)
                iprxNUM=(nROWS-iCy)*nCOLS+iCx
c               WRITE(*,*) '@PRX(bxsdy/x,iCy/x,iprxNUM):',
c    &               nROWS,nCOLS,iCy,iCx,iprxNUM,
c    &               cellLAT,cellLON,INT(gdELN)

                READ(1,'(f6.3)',REC=iprxNUM) cellPRX
                cellPRXkm=cellPRX*(60.*1.832*cosLATsrch)

                IF (cellPRXkm.EQ.0.) THEN
C                  The exception: land but with a WD...
c                  IF (gdELN<0.) CALL SLEEP(3)
                   cellNUM=rNUL
c                  (This will be a flag for Masked)
                   GOTO 889
                ELSE
C                  Other exceptions...
c                  CALL SLEEP(3)
                ENDIF
c               ------------------------------------------------

c               Shortcut if cell is outside range of data 
                IF ((CELLlat < latDATmin).OR.
     &              (CELLlat > latDATmax).OR.
     &              (CELLlon < lonDATmin).OR.
     &              (CELLlon > lonDATmax)) THEN
c                  WRITE(*,*) 'Cell is beyond the data'
                   cellNUM=0
                   GOTO 889
c                  CALL SLEEP(3)
                ENDIF

                WRITE(*,*) ''
                WRITE(*,*) 'Cell(x/y/el/px)====================',
     &                      '          ',iCx,iCy
                WRITE(*,*) 'cellLAT/LON,gdELN,cellPRXkm',
     &                      cellLAT,cellLON,INT(gdELN),cellPRXkm
c               Make chosen types of grids ----------------------------
c               Initialize outputs
                   cellVAL=rNUL
                   cellVAR=rNUL
                   cellUNC=rNUL
                   cellNUM=rnul00

c               Is there data in the cell ?   -----------------------
c               (based on if cell has data in _iXgrid.tmp)

C               nDyPT and nDxPT are the cols and rows of the data-area
C                (bigger than map area)
                nDyPT=INT((latDATmax-latDATmin)/dgCELLsz)+1
                nDxPT=INT((lonDATmax-lonDATmin)/dgCELLsz)+1

                iDyPT=INT((CELLlat-latDATmin)/dgCELLsz)+1
                iDxPT=INT((CELLlon-lonDATmin)/dgCELLsz)+1

c               Goto the cell and get the data via a pointer
                xyREC=(nDyPT-iDyPT)*nDxPT+iDxPT
C               (These are indexes over whole of data, not just output grid)
                IF (xyREC <= 0) THEN
                   WRITE(*,*)'xyREC,iDyPT,iDxPT =',xyREC,iDyPT,iDxPT
                   WRITE(*,*)'CELLlat/lon,latDATmin/max&lon&',
     &                        CELLlat,CELLlon,latDATmin,latDATmax,
     &                        lonDATmin,lonDATmax
                   CALL SLEEP(3)
                ENDIF

                READ(24,324,REC=xyREC) iDin
                IF (iDin.EQ.0) THEN
                   cellORsrchS='SRCH'
                ELSE
C                  Data in cell
                   cellORsrchS='CELL'
                ENDIF
                WRITE(*,*) 'DATinCELL:xyREC=',xyREC,' =>',cellORsrchS
324             FORMAT(i9)

C              Set the weighting parameters...
               IF (cellORsrchS.EQ.'CELL') THEN
c                 Collect the point data and store as pntX etc up to (32)
c                 This will always be by median in cell
                  weightYNS='N'

               ELSEIF (cellORsrchS.EQ.'SRCH') THEN
c                No data in cell ... 
                  IF ((interpTYPs.EQ.'MED').OR.
     &                 (interpTYPs.EQ.'AVG')) THEN
                              weightYNS='N'
                  ELSEIF (interpTYPs.EQ.'IDW') THEN
                              weightYNS='Y'
                  ELSE
                     WRITE(*,*) 'Warning: No assignment of weightYNS'
                     STOP
                  ENDIF

               ENDIF

c              Define the search radius by type, etc 
               CALL setSRCH(iCx,iCy,kmSRCHrad)

               CALL srchDATA(iCx,iCy,cellORsrchS,nnSRCH,
     &                  nbrTYPs,nnQUADs,
     &                  wghtngTYPs,weightYNS,kmSRCHrad)

C              Two choices ...
               IF (cellORsrchS.EQ.'CELL') THEN
c                 WRITE(*,*) 'Onto the interp ',interpTYPs
c                 and always use median
                  IF (iPn >= 1) THEN
                     CALL gridMED
                  ELSE
c                    WRITE(*,*) 'iPn=0'
                     GOTO 889
                  ENDIF

               ELSEIF (cellORsrchS.EQ.'SRCH') THEN
c                 If there is no data found then abort...
                  IF (iPn.EQ.0) THEN
c                   WRITE(*,*) 'No data: write nuls'
                    GOTO 889
                  ENDIF

c                 WRITE(*,*) 'Onto the interp ',interpTYPs
                  IF (INDEX(interpTYPs,'AVG')>0) THEN
c                    Average/variance/uncertainty
                     CALL gridAVG

                  ELSEIF (INDEX(interpTYPs,'MED')>0) THEN
c                    Median
                     CALL gridMED

                  ELSEIF (INDEX(interpTYPs,'IDW')>0) THEN
c                    Inverse Distance Weighting
c                    (Depending on the NN outputs of pntW may be 
c                      DIS,BTY or SLP)
                     CALL gridIDW(idwPWR)
                  ELSE
                     WRITE(*,*) 'No INTERP type specified (',
     &                             interpTYPs,')'
                     STOP
                  ENDIF
               ENDIF

c              UNCERTAINTY ----------------------------------------------
               WRITE(*,*) 'Uncertainty: ',iPARAM,iPn

c              First assign parameter details:
c              (i) The FSD range of each parameter
               DATA rngPARAM/ -99.,-99.,-99.,-99.,-99.,-99.,
     &                       -99.,-99.,-99.,-99.,-99.,
     &                       100.,100.,100.,100.,20.,6.,
     &                       -99.,100.,-99.,100.,100.,
     &                       100.,-99.,10.,4.,100.,
     &                       3500.,-99.,4.,-99.,-99./
c              WRITE(*,*) 'rngPARAM:',rngPARAM

c              Reset these variables
               measUNC=0.
               loclUNC=0.
               spatUNC=0.
               totlUNC=0.
               spatDIS=0.
               spatDSTC=0.

c              (i) Basic uncertainty (4%) even on one measurement 
               measUNC=0.04*rngPARAM(iPARAM)/2.
c              If is a calculated result,double the basic uncertainty
               IF (metaDS.EQ.'C') measUNC=measUNC*2.
               WRITE(*,*) '   Measmt: ',measUNC,' ',metaDS

c              (ii) Intra-cell uncertainty 
c              With an underlying value of 10% of PFR (Parameter Full Range)
c                (implemented mainly when cellUNC=cellVAR=0)
               loclUNC=rMAXMIN('max',0.1*rngPARAM(iPARAM)/2.
     &                               ,cellUNC,-99.)
               WRITE(*,*) '   Divrsty: (cellUNC,loclUNC)',
     &                   cellUNC,loclUNC

c              (iii) Spatial variance (based on weighted dist between 
c                  cell and nearest neighbour data points
               latCOS=COS(cellLAT*d2r)
c              WRITE(*,*) 'iPn',iPn
c              WRITE(*,*) 'pntX',(pntX(iP),iP=1,iPn)
c              WRITE(*,*) 'pntY',(pntY(iP),iP=1,iPn)
c              WRITE(*,*) 'pntW',(pntW(iP),iP=1,iPn)
c              WRITE(*,*) 'pntV',(pntV(iP),iP=1,iPn)
               DO iP=1,iPn
                  spatDIS=(pntY(iP)-cellLAT)**2. + 
     &                    ((pntX(iP)-cellLON)*latCOS)**2.
                  spatDSTC=spatDSTC+(spatDIS**0.5)*pntW(iP)
c                 (Note that here pntW is already 
c                  changed to recip and **idwPWR)
               ENDDO

c              Use a figure of 20% of PFR at rngeSVAR distance
c               and beyond that; pro-rata at less than that
               IF (gdELN>-20.) THEN
c                 Flat value in inshore (<20m WD)
                  rngeSVAR=0.05
               ELSEIF (gdELN>-40.) THEN
C                 Ramp up linearly at depths between 20 and 40m
                  rngeSVAR=0.05+0.2*(gdELN-(-20.))/(-20.)
               ELSE
C                 Plateau at depths beyond 40m
                  rngeSVAR=0.25
               ENDIF
c              rngeSVAR is the Svario range: set to
c                  0.05 (actually 7km) shallow WD; 
c                  0.25 (actually 57km) deep WD (J.Goff study)
               IF (spatDSTC>rngeSVAR) THEN
                  spatDSTC=rngeSVAR
               ENDIF
               spatUNC=0.2*rngPARAM(iPARAM)/2.*spatDSTC/rngeSVAR
               WRITE(*,*) '   Spatial: (spatDSTC,spatUNC)',
     &                   spatDSTC,spatUNC,iPn

c              (iv) Total Uncertainty:
               totlUNC=SQRT(measUNC**2+loclUNC**2+spatUNC**2)
c              Set uncertainty to a maximum of = PFR:
               IF (totlUNC > rngPARAM(iPARAM)/2.) THEN
                  totlUNC=rngPARAM(iPARAM)/2.
               ENDIF
               WRITE(*,789) measUNC,loclUNC,spatUNC,totlUNC
789            FORMAT('    TotalUNC: (meas/locl/spat/totl)',
     &             4(f6.2,' '))

c              WRITE(*,*) 'End Uncertainty CALCN'
c              ---------------------------------------------------------
c        =============================================== End Interpolation


c     OUTPUT-ER==================================================
c           Add gridcell value to grid file ---------------------------
889         CONTINUE
c           WRITE(*,*) 'Begin Output:'

106            FORMAT(f6.2,a1)
C*107            FORMAT(f6.2,a1)
108            FORMAT(f6.2,a1)
C*109            FORMAT(f6.2,a1)
111            FORMAT(f6.0,a1)
C*112            FORMAT(f6.3,a1)
c*113            FORMAT(f6.1,a1)
117            FORMAT(f8.3,a1,f8.3,a1,f6.2
     &                  ,a1,f6.2)

            if (cellNUM.EQ.0.0) THEN
c              WRITE(*,*) 'cellNUM=0 (no data)'

               WRITE(18,106) rNUL,' '
c*             WRITE(7,107) rNUL,' '
c              Cells with no output have highest uncertainty
               WRITE(8,108) rngPARAM(iPARAM)/2.,' '
c*             WRITE(9,109) rnul00,' '

            ELSEIF (cellNUM.EQ.-99.0) THEN
c              WRITE(*,*) 'cellNUM=-99 (masked)'

               WRITE(18,106) rNUL,' '
c*             WRITE(7,107) rNUL,' '
               WRITE(8,108) rNUL,' '
c*             WRITE(9,109) rnul00,' '

            ELSE
               WRITE(*,*) 'Output:',cellVAL,cellVAR,totlUNC,cellNUM
               WRITE(18,106) cellVAL,' '
c*             WRITE(7,107) cellVAR,' '
               WRITE(8,108) totlUNC,' '
c*             WRITE(9,109) cellNUM,' '

            ENDIF

c           WRITE(*,*) 'Write the topo'
               WRITE(11,111) gdELN,' '
c*             WRITE(12,112) gdSLP,' '
c*             WRITE(13,113) gdSDV,' '

c           Point data output
               WRITE(17,117) cellLAT,',',cellLON,',',cellVAL
     &                       ,',',cellPRXkm

c           Reset the output variables
               cellVAL=rNUL
               cellVAR=rNUL
               totlUNC= rngPARAM(iPARAM)
               cellNUM=0

c           ------------------------------------------------------
c           WRITE(*,*) 'End of output phase'

         ENDDO
c        (for each cell)

890      CONTINUE
      ENDDO
c     (for each row)
c     =====================================================

c     End of Program ------------------------------
      WRITE(*,*) ''
      WRITE(*,*) ''
      WRITE(*,*) 'End of program'
c     ---------------------------------------------

C     August 2007:
      CALL DATE_AND_TIME(YMDs,HMSs,Zs,DT)
      iYR=DT(1)
      iMO=DT(2)
      iDY=DT(3)
      iHR=DT(5)
      iMN=DT(6)
c     CALL GETTIM(iHR,iMN,iSC,i100)
c     CALL GETDAT(iYR,iMO,iDY)
      WRITE(16,*) 
      WRITE(16,*)'************************************'
      WRITE(16,*) 'End Time (HH MM) : Date(YY MM DD)=',
     &                   iHR,iMN,':',iDY,iMO,iYR
      WRITE(16,*)'************************************'
      WRITE(16,*) 
c     ------------------------------------------------------

c     Close the file set ---------------
      CLOSE(3)
      CLOSE(4)
      CLOSE(19)

c     Output data Grids
      CLOSE(18)
c*    CLOSE(7)
      CLOSE(8)
c*    CLOSE(9)

c     Topography Grids
      CLOSE(11)
c*    CLOSE(12)
c*    CLOSE(13)
      CLOSE(1)

c     Direct Access Files:
      CLOSE(2)
      CLOSE(14)

c     Direct Access Spatial indexes and data
      CLOSE(19)
      CLOSE(15)
      CLOSE(24)

c     Finish and close AuditReport 
      WRITE(16,*) 
      CLOSE(16)

c     Point output (monitoring) file
      CLOSE(17)
c     ----------------------------------

c: '@
c     WRITE(*,*) ''
c     WRITE(*,*) 'Normalize a grid set to the sum ? (y/.n)'
c     READ(*,*) doNORMALZEs
c     IF (doNORMALZEs.EQ.'y') THEN
c        CALL nrmlz2sum
c     ENDIF

      END
c     =========================================================




c                   SUBROUTINES                       




c     =========================================================
      SUBROUTINE ascGRIDhdrs

      CHARACTER workDIRS*25,projectS*3
      INTEGER nCOLS,nROWS

      REAL latMN,lonMN
      REAL dgCELLsz
      REAL kmSRCHradMX

      CHARACTER*3 typSRCHrad,adjstSRCHrad
      CHARACTER fnmSUFFXs*3

      COMMON /BOX/lonMN,latMN,BOXtop,boxBOT,nCOLS,nROWS
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /PROJ/projectS,workDIRS
      COMMON /RADS/dgCELLsz,kmSRCHradMX,adjstSRCHrad,typSRCHrad
      COMMON /SUFFIX/fnmSUFFXs

c     Value (output),variance,uncertainty & number (count) grids

      WRITE(*,*) ''
      WRITE(*,*) '=================================================='
      WRITE(*,*) 'Grids output to: \'
     &           ,workDIRS//projectS//'_$$$*.asc','\'

      OPEN(18,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'_.asc')
c*    OPEN(7,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'v.asc')
      OPEN(8,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'u.asc')
c*    OPEN(9,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'n.asc')

      OPEN(11,FILE=workDIRS//projectS//'_ITPb.asc')
c*    OPEN(12,FILE=workDIRS//projectS//'_ITPs.asc')
c*    OPEN(13,FILE=workDIRS//projectS//'_ITPr.asc')

c     Output of value grid as points (for faster monitoring)
      OPEN(17,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'_.txt')

c     Output value ###_
      WRITE(18,180) 'ncols        ',nCOLS
      WRITE(18,180) 'nrows        ',nROWS
      WRITE(18,181) 'xllcorner    ',lonMN
      WRITE(18,181) 'yllcorner    ',latMN
      WRITE(18,181) 'cellsize     ',dgCELLsz
      WRITE(18,183) 'NODATA_value ',rNUL
180   FORMAT(a13,i5)
181   FORMAT(a13,f10.6)
183   FORMAT(a13,f8.2)

c     Variance ###v
c*    WRITE(7,180) 'ncols        ',nCOLS
c*    WRITE(7,180) 'nrows        ',nROWS
c*    WRITE(7,181) 'xllcorner    ',lonMN
c*    WRITE(7,181) 'yllcorner    ',latMN
c*    WRITE(7,181) 'cellsize     ',dgCELLsz
c*    WRITE(7,183) 'NODATA_value ',rNUL

c     Uncertainty ###u
      WRITE(8,180) 'ncols        ',nCOLS
      WRITE(8,180) 'nrows        ',nROWS
      WRITE(8,181) 'xllcorner    ',lonMN
      WRITE(8,181) 'yllcorner    ',latMN
      WRITE(8,181) 'cellsize     ',dgCELLsz
      WRITE(8,183) 'NODATA_value ',rNUL

c     Number samples ###n
c*    WRITE(9,180) 'ncols        ',nCOLS
c*    WRITE(9,180) 'nrows        ',nROWS
c*    WRITE(9,181) 'xllcorner    ',lonMN
c*    WRITE(9,181) 'yllcorner    ',latMN
c*    WRITE(9,181) 'cellsize     ',dgCELLsz
c*    WRITE(9,183) 'NODATA_value ',rnul00

c     Elevation (-ve WD) ITPB
      WRITE(11,180) 'ncols        ',nCOLS
      WRITE(11,180) 'nrows        ',nROWS
      WRITE(11,181) 'xllcorner    ',lonMN
      WRITE(11,181) 'yllcorner    ',latMN
      WRITE(11,181) 'cellsize     ',dgCELLsz
      WRITE(11,183) 'NODATA_value ',rnul9999
c     The nul is set to except highest on-land peak

c     Seafloor slope ITPS
c*    WRITE(12,180) 'ncols        ',nCOLS
c*    WRITE(12,180) 'nrows        ',nROWS
c*    WRITE(12,181) 'xllcorner    ',lonMN
c*    WRITE(12,181) 'yllcorner    ',latMN
c*    WRITE(12,181) 'cellsize     ',dgCELLsz
c*    WRITE(12,183) 'NODATA_value ',rNUL

c     Seafloor roughness (SD) ITPR
c*    WRITE(13,180) 'ncols        ',nCOLS
c*    WRITE(13,180) 'nrows        ',nROWS
c*    WRITE(13,181) 'xllcorner    ',lonMN
c*    WRITE(13,181) 'yllcorner    ',latMN
c*    WRITE(13,181) 'cellsize     ',dgCELLsz
c*    WRITE(13,183) 'NODATA_value ',rNUL

      WRITE(17,*) 'cellLAT,cellLON,cellVAL'
     &            ,',cellPRX'

      WRITE(*,*) 'Output file headers OK'
      WRITE(*,*) 'Grid is ',nCOLS,nROWS,' in X/Y'

      END
c     =========================================================




c     =========================================================
      SUBROUTINE prepareDATA(iPARAM)
c     Interrogates dbSEABED '*_WWD.txt' file 

c     This will process the dbSEABED sample data as well as
c        any coastal classification data appended to *_WWD.txt by the
c        subroutine 'coastalCLASSN'

c     It forms a file _datMTRX.tmp (direct access) that indicates if a cell
c        has data values in it

c     It also writes indexes of sorted X and sorted Y values 
c        with pointers to the synoptic data in a direct access file

      INTEGER*4 xyREC
      INTEGER nCOLS,nROWS

      REAL pi,d2r
      REAL gdELN,gdSLP,gdSDV
      REAL latMN,lonMN
      REAL SRCHlatMN,SRCHlatMX,SRCHlonMN,SRCHlonMX,SRCHtop,SRCHbot
      REAL dgCELLsz
      REAL kmSRCHradMX
      REAL latDATmax,latDATmin,lonDATmax,lonDATmin

      CHARACTER*1 c00,c34,c39
      CHARACTER*2048 linei4S,lineo3aS,lineo3bS
      CHARACTER lineS*2048
      CHARACTER*64 vS(32)
      CHARACTER workDIRS*25,projectS*3
      CHARACTER*2048 header4S
      CHARACTER*1 metaDS
      CHARACTER*3 typSRCHrad,adjstSRCHrad
      CHARACTER*1 calibrateS
      CHARACTER fnmSUFFXs*3

      COMMON /BOX/lonMN,latMN,BOXtop,boxBOT,nCOLS,nROWS
      COMMON /ELN/gdELN,gdSLP,gdSDV,gdMAX
      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /PROJ/projectS,workDIRS
      COMMON /RADS/dgCELLsz,kmSRCHradMX,adjstSRCHrad,typSRCHrad
      COMMON /SRCH/SRCHlatMN,SRCHlatMX,SRCHlonMN,SRCHlonMX,
     &      SRCHtop,SRCHbot
      COMMON /CALIB/calibrateS
      COMMON /DATMXMN/latDATmin,lonDATmin,latDATmax,lonDATmax
      COMMON /SUFFIX/fnmSUFFXs

c     Initializations ------------------------------------
      WRITE(*,*) '>>prepareDATA: ',
     &           'c:\dbSEABED\_db9\_Mine\'//projectS//'_WWD.txt'

c     The input data file
      OPEN(4,FILE=workDIRS//projectS//'_WWD.txt')

      WRITE(*,*) 'Data holding file: |',workDIRS//
     &                      projectS//'_'//fnmSUFFXs//'d.TXT','|'
      OPEN(3,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'d.TXT')

      WRITE(*,*) 'File for calibration hold-backs (if any):'
      WRITE(*,*) '|',workDIRS// projectS//'_'//fnmSUFFXs//'d.TXT','|'
      OPEN(10,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'c.TXT')
      WRITE(*,*) ''

c     Open the file that contains the data ------------
c        (83 char direct access via spatial indexes)
      OPEN(19,FILE='C:\dbSEABED\_db9\_Interp\_iXdata.tmp',
     &   ACCESS='DIRECT',RECL=98,FORM='FORMATTED')

c     And a file to hold info on which cells have data --
c      (used for median)
      OPEN(14,FILE='C:\dbSEABED\_db9\_Interp\_datMTRX.tmp',
     &        ACCESS='DIRECT',RECL=6,FORM='FORMATTED',ERR=330)
      GOTO 3300
330      WRITE(*,*) 'Error in Direct access file dataMTRX.tmp'
         STOP
3300  CONTINUE

c     Initialize the file _datMTRX.tmp records 
c      (write map from top left corner)--
      DO iCyP=nROWS,1,-1
         DO iCxP=1,nCOLS
            xyREC=(iCyP-1)*nCOLS + iCxP
c           CJJ Aug07
            IF (xyREC>999999) THEN
               WRITE(*,*) 'xyREC>999999',xyREC
               CALL SLEEP(3)
            ENDIF
            WRITE(14,326,REC=xyREC) 000000
326         FORMAT(i6)
         ENDDO
      ENDDO
c     WRITE(*,*) 'Made DirAcc file _datMTRX.tmp:',
c    &              nROWS,nCOLS,xyREC

c     Note that some data will fall outside the array 
c     (is in search radius buffer around map area)
c     --------------------------------------------------------------

c     Write column headings --------------------------
      READ(4,567) header4S
567   FORMAT(a1024)
      WRITE(*,*) 'Input Data Header='
      WRITE(*,*) header4S(1:LEN_TRIM(header4S))
      WRITE(*,*)

      lineS=header4S
      CALL breakUpLINE(vS,lineS)

      lineo3aS=' '
      lineo3bS=' '
      DO i=1,5
         lineo3aS=lineo3aS(1:LEN_TRIM(lineo3aS))//
     &      vS(i)(1:LEN_TRIM(vS(i)))//','
      ENDDO
      lineo3aS=lineo3aS(1:LEN_TRIM(lineo3aS))//
     &        vS(iPARAM)(1:LEN_TRIM(vS(iPARAM)))//','

      lineo3bS=lineo3bS(1:LEN_TRIM(lineo3bS))//
     &        c34//'Uncert'//c34//','
      lineo3bS=lineo3bS(1:LEN_TRIM(lineo3bS))//
     &        c34//'gdELN'//c34//','
      lineo3bS=lineo3bS(1:LEN_TRIM(lineo3bS))//
     &        c34//'gdSDV'//c34//','
      lineo3bS=lineo3bS(1:LEN_TRIM(lineo3bS))//
     &        c34//'gdSLP'//c34//','
      lineo3bS=lineo3bS(1:LEN_TRIM(lineo3bS))//
     &        c34//'metaDS'//c34

      WRITE(*,*) 'Output Data Headers (###d/###c)='
      WRITE(*,304) lineo3aS(1:LEN_TRIM(lineo3aS))
      WRITE(*,304) lineo3bS(1:LEN_TRIM(lineo3bS))

      WRITE(3,304) lineo3aS(1:LEN_TRIM(lineo3aS))
     &             //lineo3bS(1:LEN_TRIM(lineo3aS))
      WRITE(10,304) lineo3aS(1:LEN_TRIM(lineo3aS))
     &             //lineo3bS(1:LEN_TRIM(lineo3bS))
304   FORMAT(a)
      WRITE(*,*) 
c     ------------------------------------------------

c     Now prepare data sub-listing -----------------------
c       (only the data in the work box (+search margin around)
      WRITE(*,*) 'Now get the data (param=',iPARAM,')...'
      WRITE(*,*) '     ReadIn:    NonNul:      Accepted:'
     &            ,'      ForCalibrn:'
      nPTS=0
      nINbx=0
      iCALIB=0
      iREC5=0

c     Will find extent of data to quicken grid scanning ---
c      (initialize these)
      latDATmin=999.
      lonDATmin=999.
      latDATmax=-999.
      lonDATmax=-999.
c     -----------------------------------------------------

      DO iD=1,10000000

         IF (MOD(iD,1000).EQ.0) THEN 
            WRITE(*,*)'iD,nPTS,nINbx,iCALIB',
     &                 iD,nPTS,nINbx,iCALIB
         ENDIF

         READ(4,568,END=677) linei4S
568      FORMAT(a1024)
c        WRITE(*,*) 'Data:',iD
c        WRITE(*,*) '   /',linei4S(1:LEN_TRIM(linei4S)),'/'

         lineS=linei4S
         CALL breakUpLINE(vS,lineS)

         rLAT=str2num(vS(1)(1:LEN_TRIM(vS(1))))
         rLON=str2num(vS(2)(1:LEN_TRIM(vS(2))))
         siteWD=str2num(vS(3)(1:LEN_TRIM(vS(3))))
         subTOP=str2num(vS(4)(1:LEN_TRIM(vS(4))))
         subBOT=str2num(vS(5)(1:LEN_TRIM(vS(5))))
         rPARAM=str2num(vS(iPARAM)(1:LEN_TRIM(vS(iPARAM))))
         ePARAM=-99
         metaDS=vS(11)(iPARAM-12+1:iPARAM-12+1)

c        Don't take null values of parameter ---------
c         (put this first for speed)
c        WRITE(*,*) 'Is the parameter nul ?',rPARAM
         IF (rPARAM.EQ.rNUL) THEN
            inBOX=0
            GOTO 678
         ELSE
c           WRITE(*,*) ''
c           WRITE(*,*) 'rLAT,rLON,siteWD,',
c    &                  'subTOP,subBOT,rPARAM,ePARAM,metaDS:',
c    &                  rLAT,rLON,INT(siteWD),
c    &                  subTOP,subBOT,rPARAM,ePARAM,metaDS
            nPTS=nPTS+1
         ENDIF
c        ---------------------------------------------

c        Does this data qualify to be written & worked on ? -------
         inbox=0
c        WRITE(*,*) 'BOXcut:lat/lonMN/MX',SRCHlatMN,SRCHlonMN,
c    &                         SRCHlatMX,SRCHlonMX
         IF ((rLAT >= SRCHlatMN).AND.(rLAT <= SRCHlatMX)) THEN
         IF ((rLON >= SRCHlonMN).AND.(rLON <= SRCHlonMX)) THEN
c           WRITE(*,*) 'Coords OK'
c           Take only the samples with subdepths that overlap the
c             box limits in subdepths...
            IF ((subTOP <= BOXbot).AND.(subBOT >= BOXtop)) THEN

               inBOX=1
               nstn=nstn+1

c              Get the topography at the point ---------------
c              Choose whether single-cell or HypoInterp
c              A  Obtain the TOPO height
ccc               CALL getTOPO(ptELN)
c              B  Obtain the HYPO height
                  CALL getHYPO(rLAT,rLON,ptELN,ptSDV,ptSLP)
c              Settle on one or the other to use in program:
c              -----------------------------------------------

            ENDIF
         ENDIF
         ENDIF
c        WRITE(*,*) 'inBOX=',inBOX
c     -------------------------------------------------------------

c        Extract some points from the data, to use them as -------
c         calibration points (set inbox=0 to take them out)
         CALL CALIBpoints(inbox,iD,rLAT,rLON,siteWD,subTOP,subBOT,
     &        rPARAM,ePARAM,ptELN,ptSDV,ptSLP,metaDS)
c        ---------------------------------------------------------

678      IF (inBOX.EQ.1) THEN
c           (ie In the box & accepted as data to be gridded)
            nINbx=nINbx+1
c           WRITE(*,*) 'Using:',iD,rLAT,rLON,INT(siteWD)
c    &             ,subTOP,subBOT

c           Add a count to the data matrix file --------
            iCyP=INT((rLAT-latMN)/dgCELLsz)+1
            iCxP=INT((rLON-lonMN)/dgCELLsz)+1
c           WRITE(*,*)iCyP,latMN,nROWS,dgCELLsz
            xyREC=0

            IF ((iCyP <= nROWS).AND.(iCxP <= nCOLS)) THEN
               IF ((iCyP >= 1).AND.(iCxP >= 1)) THEN
                  xyREC=(nROWS-iCyP)*nCOLS + iCxP
c                 WRITE(*,*) rLAT,rLON,iCyP,iCxP,xyREC

                  READ(14,323,REC=xyREC) iDATcnt
c                 WRITE(*,*) 'ReadIn iDATcnt:',xyREC,iDATcnt
                  iDATcnt=iDATcnt+1
                  WRITE(14,323,REC=xyREC) iDATcnt
c                 WRITE(*,*) 'ReWrote iDATcnt:',xyREC,iDATcnt
323               FORMAT(i6)

               ELSE
c                 WRITE(*,*) 'Data outside map: (iCxP/yP,nROWS/x)',
c    &                        iCyP,iCxP,nROWS,nCOLS
               ENDIF
            ELSE
c              WRITE(*,*) 'Data outside map: (iCxP/yP,nROWS/x)',
c    &                     iCyP,iCxP,nROWS,nCOLS
            ENDIF
c           --------------------------------------------

c           Write just position,parameter & metadata----------
c           ***_###d.txt
            WRITE(3,240) rLAT,rLON,siteWD,subTOP,subBOT,rPARAM,ePARAM,
     &                   ptELN,ptSDV,ptSLP,metaDS
240         FORMAT(2(f10.5,','),f7.1,',',2(f6.2,','),2(f6.2,','),
     &                          f7.0,',',f7.2,',',f7.2,',',a1)
c           IXIY.tmp
            iREC5=iREC5+1
            WRITE(19,243,REC=iREC5) decPT(rLAT,2),decPT(rLON,2),
     &                   siteWD,subTOP,subBOT,
     &                   rPARAM,ePARAM,-99,
     &                   ptELN,ptSDV,ptSLP,metaDS
243         FORMAT(2(f10.5,' '),f7.1,' ',2(f6.2,' '),
     &             2(f6.2,' '),i3,' ',
     &             f7.0,' ',f7.2,' ',f7.2,' ',a1,'|')

c           Find extent of data to quicken grid scanning 
            IF (rLAT < latDATmin) latDATmin=rLAT
            IF (rLAT > latDATmax) latDATmax=rLAT
            IF (rLON < lonDATmin) lonDATmin=rLON
            IF (rLON > lonDATmax) lonDATmax=rLON
C           These also have to be outside the grid and search space
cc          IF (SRCHlatMN < latDATmin) latDATmin=SRCHlatMN
cc          IF (SRCHlatMX > latDATmin) latDATmin=SRCHlatMX
cc          IF (SRCHlonMN < lonDATmin) lonDATmin=SRCHlonMN
cc          IF (SRCHlonMX < lonDATmax) lonDATmax=SRCHlonMX

         ELSE
c           WRITE(*,*) 'Out of box at:',rLAT,rLON,INT(ptELN)
         ENDIF
      ENDDO
c     ---------------------------------------------------------

677   CONTINUE

C     And report final values to screen listing:
      WRITE(*,*) 'Totals: iD,nPTS,nINbx,iCALIB=',iD,nPTS,nINbx,iCALIB
      CALL SLEEP(3)

c     Found extent of data to quicken grid scanning ----------------
      WRITE(*,*) ' '
      WRITE(*,*) 'Data box:',latDATmax
      WRITE(*,*)lonDATmin,'        ',lonDATmax
      WRITE(*,*) '         ',latDATmin
      WRITE(*,*) ''
      IF (lonDATmin.EQ.999) THEN
         WRITE(*,*) 'No data could be prepared (will STOP)'
         CALL SLEEP(3)
         STOP
      ENDIF
c     --------------------------------------------------------------

c     Closing all preparatory files ----------------
      CLOSE(15)
c     The associated TMP data file: _iXdata.tmp
      CLOSE(19)

      CLOSE(4)
      CLOSE(3)
      CLOSE(10)
      CLOSE(14)
c     ----------------------------------------------

C     Create grid-based indexes
      CALL gridINDX(nINbx+1)

      WRITE(*,*) '<<prepareDATA (',nPTS,'points;',nINbx,'used)'
      WRITE(*,*) ''

      CALL SLEEP(3)

      END
c     =========================================================

c     =========================================================
      SUBROUTINE gridINDX(N)

      TYPE xyCOORD
         INTEGER*4 iBXa
         REAL*4 Xa
         REAL*4 Ya
      END TYPE xyCOORD
      TYPE (xyCOORD) gridARR

      INTEGER*4 nDxyPT,nDxPT,nDyPT
      INTEGER*4 iDxyPT,iDxPT,iDyPT
      INTEGER*4 iD,iDin,xyREC

      REAL latDATmax,latDATmin,lonDATmax,lonDATmin
      REAL dgCELLsz,kmSRCHradMX

      CHARACTER*3 typSRCHrad,adjstSRCHrad,wghtngTYPs
      CHARACTER*1 metaDS

      COMMON /DATMXMN/latDATmin,lonDATmin,latDATmax,lonDATmax
      COMMON /RADS/dgCELLsz,kmSRCHradMX,adjstSRCHrad,typSRCHrad

C     Scans the file and puts a pointer to first data point 
C      encountered in the cell

      WRITE(*,*) '>>gridINDX:',N,'records'

      OPEN(19,FILE='C:\dbSEABED\_db9\_Interp\_iXdata.tmp',
     &   ACCESS='DIRECT',RECL=98,FORM='FORMATTED')

c     A file for storing grid spatial indexes ------------
c     Initialize it...
      OPEN(24,FILE='C:\dbSEABED\_db9\_Interp\_iXgrid.tmp',
     &   ACCESS='DIRECT',RECL=9,FORM='FORMATTED')

C       nDyPT and nDxPT are the cols and rows of the data-area
C        (bigger than map area)
        nDyPT=INT((latDATmax-latDATmin)/dgCELLsz)+1
        nDxPT=INT((lonDATmax-lonDATmin)/dgCELLsz)+1
        nDxyPT=nDxPT*nDyPT+1
        WRITE(*,*)'nDxyPT,nDyPT,nDxPT=',nDxyPT,nDyPT,nDxPT

      DO xyREC=1,nDxyPT
        WRITE(24,1843,REC=xyREC) 0
1843    FORMAT(i9)
      ENDDO
      CLOSE(24)
c     ----------------------------------------------------

c     And the data linking file 
      OPEN(15,FILE='C:\dbSEABED\_db9\_Interp\_iXlink.tmp',
     &   ACCESS='DIRECT',RECL=9,FORM='FORMATTED')

c     Initialize the file _iXlink.tmp records 
c      (one record for each data site; total N)
      DO iD=1,N
          WRITE(15,'(i9)',REC=iD) 0
      ENDDO
      WRITE(*,*) 'Initialized DirAcc file _iXlink.tmp: ',N,'records'
      CLOSE(15)

c     A file for storing grid spatial indexes ----------
      OPEN(24,FILE='C:\dbSEABED\_db9\_Interp\_iXgrid.tmp',
     &   ACCESS='DIRECT',RECL=9,FORM='FORMATTED')
c     --------------------------------------------------

c     A file for storing links between data in each cell  ----
c     This will write 28char data for later Direct Access
      OPEN(15,FILE='C:\dbSEABED\_db9\_Interp\_iXlink.tmp',
     &   ACCESS='DIRECT',RECL=9,FORM='FORMATTED')
c     --------------------------------------------------------

      WRITE(*,*)
      WRITE(*,*) 'Now spatial-indexing the data (cellwise)'

      DO iD=1,N-1
C       Scanning down the DirAcc data table...
        READ(19,1842,REC=iD)rLAT,rLON,wd,top,bot,
     &              val,uncert,numPTS,
     &             ptELN,ptSDV,ptSLP,metaDS
1842    FORMAT(2(f10.5,1x),f7.1,1x,2(f6.2,1x),
     &         2(f6.2,1x),i3,1x,
     &         f7.0,1x,f7.2,1x,f6.3,1x,a1,1x)

C       Cell coords of that data relative to max data extents...
        iDyPT=INT((rLAT-latDATmin)/dgCELLsz)+1
        iDxPT=INT((rLON-lonDATmin)/dgCELLsz)+1
c       The cell record number in #24 is...
        xyREC=(nDyPT-iDyPT)*nDxPT+iDxPT

c       WRITE(*,*) 'Reading #24:',xyREC,iDxPT,iDyPT,nDyPT
        READ(24,1843,REC=xyREC) iDin

        IF (iDin /= 0) THEN
c         WRITE(*,*) 'Some data already in cell: at ',
c    &                xyREC,': old ',iDin,' at new ',iD
          WRITE(15,1843,REC=iD) iDin
          WRITE(15,1843,REC=iDin) -1
C         Ie. Write old dataloc at new dataloc in #15,
C            -1 at old dataloc in #15 (functions as end of links),
C            and new data iD at cell in #24. 
          WRITE(24,1843,REC=xyREC) iD

        ELSE
c         WRITE(*,*) 'First data in cell: iD,iDxyPT',iD,iDxyPT
c         Write data #15 record (-1=flag for sole data - no link)...
          WRITE(15,1843,REC=iD) -1
c         and cell record at #24...
          WRITE(24,1843,REC=xyREC) iD

C         AND write it to #16 too for debugging
c         WRITE(16,*) 'datINcell:',iDxyPT,iDyPT,iDxPT,'a'
        ENDIF

184   ENDDO

      CLOSE(19)

      CLOSE(24)
      CLOSE(15)

C     Note: These structures are now used by srchDATA

      END
c     =========================================================




c     =========================================================
      SUBROUTINE srchDATA(iCx,iCy,cellORsrchS,nnSRCH,
     &            nbrTYPs,nnQUADs,
     &            wghtngTYPs,weightYNS,kmSRCHrad)

C     This is called for each cell

      INTEGER nCOLS,nROWS
      INTEGER*1 iPQ(4),iPQn(4)
      INTEGER*4 nDxyPT,nDxPT,nDyPT
      INTEGER*4 iDxyPT,iDxPT,iDyPT
      INTEGER*4 iDin,xyREC

      REAL ptW(4,8),ptY(4,8),ptX(4,8),ptZ(4,8),ptV(4,8),
     &     ptELNav(4,8),ptELNsd(4,8),ptELNsl(4,8)
      REAL pntW(32),pntY(32),pntX(32),pntZ(32),pntQ(32),pntV(32),
     &     pntELNav(32),pntELNsd(32),pntELNsl(32)
      REAL dgCELLsz,kmSRCHrad,kmSRCHradMX
      REAL latCOS
      REAL kmDSTC,kmDSTCx,kmDSTCy,dgDSTC
      REAL pi,d2r
      REAL gdELN,gdSLP,gdSDV
      REAL idwDSTC
      REAL latMN,lonMN
      REAL latDATmax,latDATmin,lonDATmax,lonDATmin
      REAL cellSRCHrad,cellDISTrad

      CHARACTER*3 typSRCHrad,adjstSRCHrad,wghtngTYPs
      CHARACTER workDIRS*25,projectS*3
      CHARACTER*1 c00,c34,c39
      CHARACTER*1 metaDS
      CHARACTER cellORsrchS*4,weightYNS*1
      CHARACTER*3 nnQUADs,nbrTYPs
      CHARACTER*1 stopSPIRALs

      COMMON /BOX/lonMN,latMN,BOXtop,boxBOT,nCOLS,nROWS
      COMMON /ELN/gdELN,gdSLP,gdSDV,gdMAX
      COMMON /CELL/cellLAT,cellLON
      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /DSTC/kmDSTC,kmDSTCx,kmDSTCy,dgDSTC
      COMMON /GRD/iPn,inCELL,inSRCH
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /PNT/pntW,pntY,pntX,pntZ,pntQ,pntV,
     &        pntELNav,pntELNsd,pntELNsl,metaDS
      COMMON /PROJ/projectS,workDIRS
      COMMON /RADS/dgCELLsz,kmSRCHradMX,adjstSRCHrad,typSRCHrad
      COMMON /DATMXMN/latDATmin,lonDATmin,latDATmax,lonDATmax

c     ------------------------------------------------------
c.....Finds all the nearby points using a spiral

c     WRITE(*,*) '>>srchDATA'
c     WRITE(*,*) '>>srchDATA(typSRCHrad,dgCELLsz,kmSRCHradMX,',
c    &                        'cellORsrchS,nnSRCH):'
c     WRITE(*,*) '   ',typSRCHrad,dgCELLsz,kmSRCHradMX,'  ',
c    &                        cellORsrchS,nnSRCH
c     WRITE(*,*)

c       Will need nDxyPT later ...
        nDyPT=INT((latDATmax-latDATmin)/dgCELLsz)+1
        nDxPT=INT((lonDATmax-lonDATmin)/dgCELLsz)+1
        nDxyPT=nDxPT*nDyPT+1
c       WRITE(*,*)'nDxyPT,nDyPT,nDxPT=',nDxyPT,nDyPT,nDxPT

      latCOS=COS(((cellLAT+rLAT)/2.)*d2r)
c     WRITE(*,*) 'latCOS,d2r',latCOS,d2r

c     -------------------------------------------------------
c     Initialize for the cell ------------
      DO iQ=1,4
         DO iP=1,8
            ptW(iQ,iP)=rNUL
            ptX(iQ,iP)=-999.
            ptY(iQ,iP)=-999.
            ptZ(iQ,iP)=rnul9999
            ptV(iQ,iP)=rNUL
            ptELNav(iQ,iP)=rnul9999
            ptELNsd(iQ,iP)=rNUL
            ptELNsl(iQ,iP)=rNUL
         ENDDO
         iPQ(iQ)=0
         iPQn(iQ)=0
      ENDDO
c     WRITE(*,*) 'Initialized pt* arrays'
         DO iP=1,4*8
            pntW(iP)=rNUL
            pntX(iP)=-999.
            pntY(iP)=-999.
            pntZ(iP)=rnul9999
            pntQ(iP)=rNUL
            pntV(iP)=rNUL
            pntELNav(iP)=rnul9999
            pntELNsd(iP)=rNUL
            pntELNsl(iP)=rNUL
         ENDDO
      iQ=0
      iP=0
      iPn=0
c     WRITE(*,*) 'Initialized pnt* arrays'

         numDATin=0

c     'Define what to do if a difference' is zero ---------------------
c     (100m distance, 1m depth difference, 1% gradient;
c      these are ~ accuracies of measurement at sea)
      rMINdis=0.1
      fFACT=1000.

      rMINxyz=SQRT((0.001*fFACT)**2+2*(0.1**2))
c     rMINxyz sets minimum allowed value
c     (Base case with fFACT=1 is 1m2+100m2+100m2 in km2)

      rMINbtr=0.01
c     Units: / dis km / xyz NMNL km [cubrt(km^2.m)] / btr RATIO [1m/m] / 

C =======================================================================
c     How many cells around should it search ?
      if (cellORsrchS.EQ.'CELL') THEN
        cellSRCHrad=0.5
      ELSE
c       Will need to gather data from cells around...
        cellSRCHrad=kmSRCHrad/(60.*1.832)/dgCELLsz
        WRITE(*,*) 'Search: ',kmSRCHrad,'km ',cellSRCHrad,'cells'
      ENDIF

C     Data stock coordinates of the cell
      iDyPT=INT((CELLlat-latDATmin)/dgCELLsz)+1
      iDxPT=INT((CELLlon-lonDATmin)/dgCELLsz)+1
c     WRITE(*,*)iDxPT,iDyPT

c     WRITE(*,*) 'Entering spiral'
      iRUN=0
      iLENGTH=0
      iSPIRAL=-1
      iDcellX=iDxPT
      iDcellY=iDyPT
      stopSPIRALs='n'

      DO WHILE (stopSPIRALs /= 'y') 

c       If it is proximity ('DST') search then the condition is: 
c        radial distance (in cells) is reached from target gridcell
c       If it is nearest neighbour ('NEA') search then condition is: 
c        that a set number of points have been found

c       Set up a square spiral ---------------------
c       (iRUN is path number
c         iLENGTH is cell number along run
c         iSPIRAL is whether to apply iLENGTH to x or y)

          iRUN=iRUN+1
          xAXIS=MOD(iRUN,2)
          yAXIS=MOD(iRUN+1,2)

        IF (MOD(iRUN,2).EQ.1) THEN
          iLENGTH=iLENGTH+1
          iSPIRAL=iSPIRAL*(-1)
        ENDIF
c       --------------------------------------------

        DO iMOVE=1,iLENGTH

          IF (cellORsrchS.EQ.'SRCH') THEN
            iDcellX=iDcellX+iSPIRAL*1.*xAXIS
            iDcellY=iDcellY+iSPIRAL*1.*yAXIS
c           WRITE(*,*) 'SRCH',iDcellX,iDcelly,iMOVE,'|',
c    &               iRUN,iLENGTH,iSPIRAL,xAXIS,yAXIS
          ELSEIF (cellORsrchS.EQ.'CELL') THEN
c           Enact an aborting for cellORsrch='CELL'
c           iDcellX and iDcellY are unchanged
            stopSPIRALs='y'
c           WRITE(*,*) 'stopSPIRALs=y (data in cell)'
C           (This will still collect all the points in the cell)
          ENDIF

C         And end spiral when iLENGTH > cellSRCHrad
          IF (REAL(iLENGTH)/2. > cellSRCHrad ) THEN
            stopSPIRALs='y'
c           WRITE(*,*) 'stopSPIRALs=y (too_distant)'
          ENDIF

C         Which cells are within the search radius ? 
          cellDISTrad=(iDcellY-iDyPT)**2+(iDcellX-iDxPT)**2
          cellDISTrad=cellDISTrad**0.5
          IF (nbrTYPs.EQ.'DST') THEN
            IF (cellDISTrad > cellSRCHrad) THEN
c              Don't do this cell: go on to next 
c              WRITE(*,*) 'Cell too distant',
c    &                     cellDISTrad,cellSRCHrad,'cells'
               GOTO 665
            ENDIF
          ENDIF

C         Are any cell coords -ve for xyREC ?
          xyREC=(nDyPT-iDcellY)*nDxPT+ iDcellX
          IF (xyREC <= 0) THEN
c           WRITE(*,*) 'Halo past data space'
            GOTO 665
          ENDIF

c         Goto the cell and get the data via a pointer
          xyREC=(nDyPT-iDcellY)*nDxPT+ iDcellX
C         (These are indexes over whole of data, not just output grid)

C         What is the data index in the cell (via DirAccs) ?
C         (this enters the linkages, or points to only data)
          READ(24,2844,ERR=911,REC=xyREC) iDin
2844      FORMAT(i9)
          GOTO 912

911       WRITE(16,*) 'Problem at #24: xyREC,iDcellX/Y=',
     &                 xyREC,iDcellX,iDcellY
          WRITE(16,*)'   nDxyPT,nDyPT,nDxPT=',nDxyPT,nDyPT,nDxPT
          WRITE(16,*)'   CELLlat/lon=',CELLlat,CELLlon
          WRITE(16,*)'   latDATmin/max,latDATmin/max=',
     &                  latDATmin,latDATmax,lonDATmin,lonDATmax
          WRITE(*,*) 'Problem reading DA at #24'
          GOTO 665

912       CONTINUE
c         WRITE(*,*)'Data iDin=',iDin,' at ',xyREC,'and links'

C         Then need to search all data records in each cell...
          DO WHILE (iDin /= -1) 
C           (-1 in DA #15 file signifies end of data or data links)

            IF (iDin.EQ.0) GOTO 665
C           Exit from search of links if iDin=0 (no data in cell)

C           Get the data from the data DA #19 file...
            READ(19,2842,ERR=913,REC=iDin)rLAT,rLON,wd,top,bot,
     &                            val,uncert,numPTS,
     &                            ptELN,ptSDV,ptSLP,metaDS
2842        FORMAT(2(f10.5,1x),f7.1,1x,2(f6.2,1x),
     &             2(f6.2,1x),i3,1x,
     &             f7.0,1x,f7.2,1x,f6.3,1x,a1,1x)
            GOTO 914

913         WRITE(*,*) 'Problem reading #19'
            WRITE(16,*) 'Problem at #19: iDin=',iDin
            GOTO 665

914         numDATin=numDATin+1
c           WRITE(*,*)'Fetched point:',rLAT,rLON,wd,val,numDATin

C           Has full number of points been reached ?
            IF ((nbrTYPs.EQ.'NEA').AND.(numDATin >= nnSRCH)) THEN
               stopSPIRALs='y'
c              WRITE(*,*) 'stopSPIRALs=y (>',nnSRCH,' points)'
            ENDIF

C           For link to next data: use DA #15 read...
            READ(15,2844,ERR=915,REC=iDin) iDin2
            GOTO 916

915         WRITE(16,*) 'Problem at #15: iDin=',iDin
            WRITE(*,*) 'Problem reading #15'
            GOTO 665

916         iDin=iDin2
C           (If this iDin is -1 then DO WHILE cuts in)

C           Is the data inCELL or just nearby ?
            IF ((iDcellY.EQ.iDyPT).AND.(iDcellX.EQ.iDxPT)) THEN
               inCELL=1
               stopSPIRALs='y'
               WRITE(*,*) 'stopSPIRALs=y (data in cell)'
            ELSE
               inSRCH=1
            ENDIF

c           Form the sets of data for interpolator to work on ----------
            IF ((inSRCH.EQ.1).OR.(inCELL.EQ.1)) THEN
c             WRITE(*,*) 'inCELL/inSRCH:',
c    &                      inCELL,inSRCH,iCxPT,iCyPT
c             WRITE(*,*) '    rLAT,rLON,wd,val:',
c    &                      rLAT,rLON,INT(wd),val

C             Calculate distance for weightings
              dgDSTCx2=((cellLON-rLON)*latCOS)**2.
              dgDSTCy2=(cellLAT-rLAT)**2
              dgDSTC=SQRT(dgDSTCx2+dgDSTCy2)
              kmDSTC=dgDSTC*60.*1.832

c             Identify quad of the data relative to cell
              IF (nnQUADs.EQ.'QAD') THEN
                IF ((rLAT > cellLAT).AND.(rLON >= cellLON)) iQ=1
                IF ((rLAT <= cellLAT).AND.(rLON >= cellLON)) iQ=2
                IF ((rLAT <= cellLAT).AND.(rLON < cellLON)) iQ=3
                IF ((rLAT > cellLAT).AND.(rLON < cellLON)) iQ=4
              ELSE
C               Use no quads (ie iQ always=1 )
                iQ=1
              ENDIF
c             WRITE(*,*) 'Quad=',iQ

c             ------------------------------------------------
c             Calculate the weighting (options set for project)
              IF (wghtngTYPs.EQ.'DIS') THEN
C                DIStance
                 idwDSTC=ABS(kmDSTC)

                 IF (idwDSTC < rMINdis) idwDSTC=rMINdis
c                WRITE(*,*) 'Weighting DIS 2',idwDSTC,kmDSTC,rMINdis

              ELSEIF (wghtngTYPs.EQ.'XYZ') THEN
C                Geographic distance (XY) and WD (Z)
c                WRITE(*,*) 'Weighting XYZ 1', ABS(gdELN-ptELN)

c                (The unit is km; where m WD has a 1000x exaggeration
c                 at fFACT=1, 10x at fFACT=10 etc)
c                (Mostly use 1000 factor in favour of ELN; m:km)

                 dpDSTC=ABS(gdELN-ptELN)/1000.*fFACT
c                dpDSTC=ABS((gdELN-ptELN)/gdELN)

c                Make a combined Euclidean distance
                 idwDSTC=SQRT(dpDSTC**2+kmDSTC**2)

                 IF (idwDSTC < rMINxyz) idwDSTC=rMINxyz
c                WRITE(*,*) 'Weighting XYZ 2',
c    &                       idwDSTC,dpDSTC,kmDSTC,rMINxyz

              ELSEIF (wghtngTYPs.EQ.'BTR') THEN
c                WRITE(*,*) 'Weighting BTR 1', ABS(gdELN-ptELN)
                 idwDSTC=ABS((gdELN-ptELN)/gdELN)
                 IF (idwDSTC < rMINbtr) idwDSTC=rMINbtr
c                WRITE(*,*) 'Weighting BTR 2',idwDSTC,gdELN,ptELN,rMINbtr

              ELSE
                 WRITE(*,*) 'Bad weighting method (',wghtngTYPs,')'
                 STOP

              ENDIF
c             ------------------------------------------------

c             iPQ is the total of points encountered in search in that quad
c             iPQn is number in that quad buffer at present
              iPQ(iQ)=iPQ(iQ)+1
              iPQn(iQ)=iPQn(iQ)+1
c             iQ is already known

c             If iPQ > 8 new data may displace old 
c             (if its idwDSTC is small enough)
c             and buffer stays at size nnSRCH
              IF (iPQn(iQ) > 8) THEN
c               WRITE(*,*) 'Quad point buffer (8) exceeded:',
c    &                      iPQ(iQ),iQ
                iPQn(iQ)=8
              ENDIF

c             WRITE(*,*) 'inSRCH,inCELL,iP,iPQn(iQ),nnSRCH=',
c    &                        inSRCH,inCELL,iP,iPQn(iQ),nnSRCH

c             (If new one closer than any of present NN 
c                then insert into NN set)

c              Loop through iPQn(iQ) set to find if new
c                one is less distant than another
c              WRITE(*,*) 'BeforeSort(ptW/V)',iPQn(iQ),iQ
c              WRITE(*,*) (ptW(iQ,iPw),iPw=1,iPQn(iQ))
c              WRITE(*,*) (ptV(iQ,iPw),iPw=1,iPQn(iQ))
               DO iPw=1,iPQn(iQ)
                  IF ((idwDSTC < ptW(iQ,iPw)).OR.
     &                     (ptV(iQ,iPw).EQ.rNUL)) THEN
c                    Can we shuffle without losing a quad's data entirely ?
c                    WRITE(*,*) 'Found first one at  >=  distance or -99:',
c    &                           ptW(iQ,iPw),' >= ',kmDSTC,'or -99'
c                    WRITE(*,*) 'Insert into the pack at',iPQ(iQ)
c                    First move the pack to make space
                     DO iPs=iPQn(iQ)-1,iPw,-1
                        ptW(iQ,iPs+1)=ptW(iQ,iPs)
                        ptX(iQ,iPs+1)=ptX(iQ,iPs)
                        ptY(iQ,iPs+1)=ptY(iQ,iPs)
                        ptZ(iQ,iPs+1)=ptZ(iQ,iPs)
                        ptV(iQ,iPs+1)=ptV(iQ,iPs)
                        ptELNav(iQ,iPs+1)=ptELNav(iQ,iPs)
                        ptELNsd(iQ,iPs+1)=ptELNav(iQ,iPs)
                        ptELNsl(iQ,iPs+1)=ptELNav(iQ,iPs)
                     ENDDO

c                    And slot the new one in at iPw (now spare)

c                    (This is where the type of distance is set)
                     ptW(iQ,iPw)=idwDSTC
                     ptX(iQ,iPw)=rLON
                     ptY(iQ,iPw)=rLAT
                     ptZ(iQ,iPw)=wd
                     ptV(iQ,iPw)=val
                     ptELNav(iQ,iPw)=ptELN
                     ptELNsd(iQ,iPw)=ptSDV
                     ptELNsl(iQ,iPw)=ptSLP
                     GOTO 862
                  ELSE
c                    Pack unchanged
                  ENDIF
               ENDDO
862            CONTINUE

c              WRITE(*,*) 'AfterSort (ptW/V)',iPQn(iQ),iQ
c              WRITE(*,*) (ptV(iQ,iPw),iPw=1,iPQn(iQ))
c              WRITE(*,*) (ptW(iQ,iPw),iPw=1,iPQn(iQ))

             ENDIF
c           (For inSRCH/inCELL=1)

          ENDDO
C         End of loop to detect linked data

665     ENDDO
C       665 signifies no data in a scanned cell (iDin=0)

      ENDDO
C     End of loop to create spiral around target cell

C =======================================================================
C     Now write them to an array that will be used for IDW, etc
      iP=0
      iPn=0
c     (index and total number respectively
c      scrolling up through higher ptW)
c     Assemble a set of 8 points max
      DO iPw=1,8
c        For each quad add one in turn
         DO iQ=1,4
            IF (ptV(iQ,iPw) /= rNUL) THEN
               iP=iP+1
               iPn=iPn+1
               pntW(iP)=ptW(iQ,iPw)
               pntX(iP)=ptX(iQ,iPw)
               pntY(iP)=ptY(iQ,iPw)
               pntZ(iP)=ptZ(iQ,iPw)
               pntQ(iP)=iQ
               pntV(iP)=ptV(iQ,iPw)
               pntELNav(iP)=ptELNav(iQ,iPw)
               pntELNsd(iP)=ptELNsd(iQ,iPw)
               pntELNsl(iP)=ptELNsl(iQ,iPw)
            ENDIF
         ENDDO
      ENDDO
c              ----------------------------------------------

c     Need to limit this to the pack size after the sort ----
C     Set those above the limit to null values
      IF (iPn > nnSRCH) THEN
         WRITE(*,*) 'Cull to nnSRCH number:',
     &      iPn,' to',nnSRCH
         DO iP=iPn+1,nnSRCH
            pntW(iP)=rNUL
            pntX(iP)=-999.
            pntY(iP)=-999.
            pntZ(iP)=rnul9999
            pntV(iP)=rNUL
            pntQ(iP)=rNUL
            pntELNav(iP)=rnul9999
            pntELNsd(iP)=rNUL
            pntELNsl(iP)=rNUL
         ENDDO
         iPn=nnSRCH
      ENDIF
c              ----------------------------------------------

c     Report the results ---------------------------------
      IF (iPn >= 1) THEN
c        WRITE(*,*) '<<srchDATA:(iQ,iPn,nnSRCH)',iPn,nnSRCH
c        WRITE(*,*) '   (typSRCHrad,kmSRCHrad)'
c        WRITE(*,*) '   ',typSRCHrad,kmSRCHrad
c        WRITE(*,287) '    pntX:',(pntX(iP),iP=1,iPn)
c        WRITE(*,287) '    pntY:',(pntY(iP),iP=1,iPn)
         WRITE(*,287) '    pntV:',(pntV(iP),iP=1,iPn)
         WRITE(*,287) '    pntW:',(pntW(iP),iP=1,iPn)
         WRITE(*,288) '    pntQ:',(INT(pntQ(iP)),iP=1,iPn)
287      FORMAT(a9,24f7.2) 
288      FORMAT(a9,24i7) 
      ENDIF
c     ----------------------------------------------------

c     For the inCELL data no weights are used
      IF (weightYNS.EQ.'N') THEN
c        Will need only the set of points all weighted to 1.0
         DO iP=1,iPn
            pntW(iP)=1.
         ENDDO
      ELSEIF (weightYNS.EQ.'Y') THEN
c        Will use weighted and sorted values
      ENDIF

      END




c     =========================================================
      SUBROUTINE gridAVG 
c     Makes most primitive grids-average and variance

      REAL pntW(32),pntY(32),pntX(32),pntZ(32),pntQ(32),pntV(32),
     &     pntELNav(32),pntELNsd(32),pntELNsl(32)
      REAL pi,d2r

      CHARACTER*1 c00,c34,c39
      CHARACTER*1 metaDS

      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /GRD/iPn,inCELL,inSRCH
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /OUT/cellVAL,cellVAR,cellUNC,cellNUM
      COMMON /PNT/pntW,pntY,pntX,pntZ,pntQ,pntV,
     &        pntELNav,pntELNsd,pntELNsl,metaDS

c     WRITE(*,*) '>>gridAVG',iPn
c     WRITE(*,*) 'pntV:',(pntV(iPw),iPw=1,iPn)

      pntVALsum=0.
      pntVALsum2=0.

      cellAVG=rNUL
      cellVNC=rNUL

      DO iP=1,iPn
         IF (pntV(iP) /= rNUL) THEN
            pntVALsum=pntVALsum+pntV(iP)
         ENDIF
      ENDDO
      cellAVG=pntVALsum/REAL(iPn)

      DO iP=1,iPn
         IF (pntV(iP) /= rNUL) THEN
            pntVALsum2=pntVALsum2+((pntV(iP)-cellAVG))**2
         ENDIF
c        'And by the way set all pntW() to 1/iPn
C           (affects uncertainty distance weightings)
         pntW(iP)=1./REAL(iPn)
      ENDDO
      cellVNC=pntVALsum2/REAL(iPn)

c     Re-assign to general names
      cellVAL=cellAVG
      cellVAR=cellVNC
c     Use Standard Error of Mean for AVG uncertainty (but as SD)
      cellUNC=SQRT(cellVNC/REAL(iPn))
      cellNUM=REAL(iPn)

449   WRITE(*,*) '<<gridAVG (iPn)',iPn
      WRITE(*,*) 'cellVAL/VAR/UNC:',cellVAL,cellVAR,cellUNC

      END
c     =========================================================




c     =========================================================
      SUBROUTINE gridMED 

      REAL pntW(32),pntY(32),pntX(32),pntZ(32),pntQ(32),pntV(32),
     &     pntELNav(32),pntELNsd(32),pntELNsl(32)
      REAL pi,d2r

      CHARACTER*1 c00,c34,c39
      CHARACTER*1 metaDS

      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /GRD/iPn,inCELL,inSRCH
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /OUT/cellVAL,cellVAR,cellUNC,cellNUM
      COMMON /PNT/pntW,pntY,pntX,pntZ,pntQ,pntV,
     &        pntELNav,pntELNsd,pntELNsl,metaDS
c     Assigns a median value to the cell

c     WRITE(*,*) '>>gridMED',iPn
c     WRITE(*,*) 'pntV:',(pntV(iPw),iPw=1,iPn)
c     pntV is limited to size (32)

      IF (iPn <= 2) THEN
         XMED=pntV(1)
      ELSE
c        This median finder from Numerical recipes
c         (It is efficient for small N)
         CALL sortN(iPn,pntV)
         N2=iPn/2
         IF (2.*N2.EQ.iPn) THEN
            XMED=0.5*(pntV(N2)+pntV(N2+1))
         ELSE
            XMED=pntV(N2+1)
         ENDIF
      ENDIF

c     And calc the SD for uncertainty ------------------------
c     StdErrMedn=1.253*SD/N**2 (see Hyperstat Online)
      pntVALsum=0.
      pntVALsum2=0.

      cellAVG=rNUL
      cellVNC=rNUL

      DO iP=1,iPn
         IF (pntV(iP) /= rNUL) THEN
            pntVALsum=pntVALsum+pntV(iP)
         ENDIF
c        'And by the way set all pntW() to 1/iPn
C           (affects uncertainty distance weightings)
         pntW(iP)=1./REAL(iPn)
      ENDDO
      cellAVG=pntVALsum/REAL(iPn)

c     Initialize min/max (more extreme than any parameter value)
      pntVmax=-999
      pntVmin=999
      DO iP=1,iPn
         IF (pntV(iP) /= rNUL) THEN
            pntVALsum2=pntVALsum2+((pntV(iP)-cellAVG))**2

C           Also find min,max values (for Err in Medn
            IF (pntV(iP)>pntVmax) pntVmax=pntV(iP)
            IF (pntV(iP)<pntVmin) pntVmin=pntV(iP)

         ENDIF
      ENDDO
      cellVNC=pntVALsum2/REAL(iPn)
c     --------------------------------------------------------

c     Re-assign to general names
      cellVAL=XMED
c     Use Standard Deviation=1/3 range
      cellVAR=((pntVmax-pntVmin)/3.)**2

c     WRITE(*,*) 'pntVmin/max,cellVAR:',pntVmin,pntVmax,cellVAR
      cellUNC=1.253*(cellVAR/REAL(iPn))*0.5
      cellNUM=REAL(iPn)

c     WRITE(*,*) '<<gridMED (iPn)',iPn
c     WRITE(*,*) '   cellVAL/VNC/VAR/UNC:',
c    &               cellVAL,cellVNC,cellVAR,cellUNC

      END
c     =========================================================




c     =======================================================================
      SUBROUTINE gridIDW(idwPWR)
c     Makes most commonly used grid INVERSE DISTANCE WEIGHTED

      REAL pntW(32),pntY(32),pntX(32),pntZ(32),pntQ(32),pntV(32),
     &     pntELNav(32),pntELNsd(32),pntELNsl(32)
      REAL pi,d2r
      CHARACTER*1 c00,c34,c39
      CHARACTER*1 metaDS

      COMMON /GRD/iPn,inCELL,inSRCH
      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /OUT/cellVAL,cellVAR,cellUNC,cellNUM
      COMMON /PNT/pntW,pntY,pntX,pntZ,pntQ,pntV,
     &        pntELNav,pntELNsd,pntELNsl,metaDS

c     WRITE(*,*) '>>gridIDW',iPn,-idwPWR

      totWGT=0.
      cellIDW=0.
      cellVAR=0.

c     First get total weighting ---------------
c     WRITE(*,*) 'Calc TotWeights'
      DO iP=1,iPn
         IF (pntW(iP).EQ.rNUL) THEN
            WRITE(*,*) 'pntW=-99 <E>'
         ELSE
            totWGT=totWGT+(pntW(iP)**(-idwPWR))
         ENDIF
c        WRITE(*,*) 'iPn/pntW/totWGT:',iPn,pntW(iP),totWGT
      ENDDO
c     -----------------------------------------

c     Then calculate IDW and weighted value per point ------------------------
c     WRITE(*,*) 'Calc pntW weights'
      DO iP=1,iPn
         IF (pntW(iP).EQ.rNUL) THEN
            pntW(iP)=0.
         ELSE
C           (This is the actual IDW)
            pntW(iP)=(pntW(iP)**(-idwPWR))/totWGT
c           pntW is now normalized to sumWeights
         ENDIF
         cellIDW=cellIDW+pntW(iP)*pntV(iP)
c        WRITE(*,*) 'pntV/W(norm)/cellIDW',
c    &               pntV(iP),pntW(iP),cellIDW
      ENDDO
c     -----------------------------------------------------

c     Then calculate weighted variance per point -----------
c     WRITE(*,*) 'Calc Variance'
      DO iP=1,iPn
         cellVAR=cellVAR+pntW(iP)*(pntV(iP)-cellIDW)**2
c        WRITE(*,*) 'pntV/W,cellVAR',
c    &               pntV(iP),pntW(iP),cellVAR
      ENDDO
c     -----------------------------------------------------

c     Re-assign to general names
c     WRITE(*,*) 'Finishing up'
      cellVAL=cellIDW
      cellUNC=SQRT(cellVAR)
c     (Like StdErrMean; N=1 because pntW is normalized)
      cellVAR=cellVAR
c     (cellVAR is already set above)
      cellNUM=REAL(iPn)

C     WRITE(*,*) '<<gridIDW:(cellVAL/VAR/UNC)'
c     WRITE(*,*) cellVAL,cellVAR,cellUNC
c     CALL SLEEP(3)

      END
c     =========================================================




c     =========================================================
      SUBROUTINE CALC3x3

      INTEGER*2 iTOPO(10800)
      REAL gdELN,gdSLP,gdSDV
      REAL gELN(3,3)
      REAL xSB(3,3),ySB(3,3)
      REAL pi,d2r

      CHARACTER*1 c00,c34,c39

      COMMON /ELN/gdELN,gdSLP,gdSDV,gdMAX
      COMMON /CELL/cellLAT,cellLON
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /CONST/pi,d2r,c00,c34,c39

c     WRITE(*,*) '>>calc3x3 incl Sobel:'

c     WRITE(*,*) 'cellLAT/LON=',cellLAT,cellLON
      cosLAT=COS(cellLAT*d2r)
c     (Will need to pad out for ends of the earth one day)

c     NOTE: gELN & TOPO are elevations (land=+ve)
c           until the final output

c     Sobel filter parameters --------------------------------------
      DATA xSB/-1.,+0.,+1.,
     &         -2.,+0.,+2.,
     &         -1.,+0.,+1./
      DATA ySB/-1.,-2.,-1.,
     &         +0.,+0.,+0.,
     &         +1.,+2.,+1./

c     Clockwise notation
c               5   6   7
c               4   x   0
c               3   2   1
c     --------------------------------------------------------------

      sumELN=0.
      sumELN2=0.
      ySLP=rNUL
      xSLP=rNUL
      gdELN=rnul9999
      gdSLP=rNUL
      gdSDV=rNUL

c     Get the 3x3 set
c     These cell coords in ETOPO2 are needed
      DO iR=1,3
         iROW=INT((90.-cellLAT)*30.+1.)+(iR-2)
c        Get the values from the Direct Access ETOPO2 file
         READ(2,REC=iROW) (iTOPO(iW),iW=1,10800)
         DO iC=1,3
            iCOL=INT((180.+cellLON)*30.+1.)+(iC-2)
            gELN(iR,iC)=REAL(iTOPO(iCOL))
         ENDDO
      ENDDO
c     WRITE(*,*) 'SOBEL slopes: (ELN vals)'
c     WRITE(*,*) (INT(gELN(1,iC)),iC=1,3)
c     WRITE(*,*) (INT(gELN(2,iC)),iC=1,3)
c     WRITE(*,*) (INT(gELN(3,iC)),iC=1,3)

c     Calculate sum
      DO iC=1,3
         DO iR=1,3
            sumELN=sumELN+gELN(iR,iC)
         ENDDO
      ENDDO
      avgELN=sumELN/9.
      gdELN=avgELN
c     (But may make gdELN the HYPO interp later)

c     Calculate SD
      DO iC=1,3
         DO iR=1,3
            sumELN2=sumELN2+(gELN(iR,iC)-avgELN)**2
         ENDDO
      ENDDO
      gdSDV=SQRT(sumELN2/9.)

c     Calculate Sobel X,Y slopes -------------------------
c     (gdELN subtracted brings all down to common bias)
c     WRITE(*,*) 'Calc SOBEL slopes:'
c     WRITE(*,*) (xSB(1,iC),iC=1,3),(ySB(1,iC),iC=1,3)
c     WRITE(*,*) (xSB(2,iC),iC=1,3),(ySB(2,iC),iC=1,3)
c     WRITE(*,*) (xSB(3,iC),iC=1,3),(ySB(3,iC),iC=1,3)

      xSLP=gELN(1,1)*xSB(1,1)   +gELN(1,3)*xSB(1,3)
     &    +gELN(2,1)*xSB(2,1)   +gELN(2,3)*xSB(2,3)
     &    +gELN(3,1)*xSB(3,1)   +gELN(3,3)*xSB(3,3)
      xSLP=xSLP/8.

      ySLP=gELN(1,1)*ySB(1,1)+gELN(1,2)*ySB(1,2)+gELN(1,3)*ySB(1,3)
c    &                                                             
     &    +gELN(3,1)*ySB(3,1)+gELN(3,2)*ySB(3,2)+gELN(3,3)*ySB(3,3)
      ySLP=ySLP/8.

      gdSLP1=SQRT(xSLP**2+ySLP**2)

c     And normalize to grid size of 2min X&Y
      gdSLP2=SQRT((2./60.)**2+(2./60.*cosLAT)**2)*1832.*60.
      gdSLP=gdSLP1/gdSLP2
c     WRITE(*,*) 'gdSLP=',gdSLP,d2r,ATAN(gdSLP)
      IF (gdSLP /= 0.) THEN
         iangSLP=INT(ATAN(gdSLP)/d2r+0.5)
      ELSE
         iangSLP=90
      ENDIF

c     WRITE(*,*) 'Sobel outputs: (xSLP,ySLP,gdSLP=)',
c    &                      xSLP,ySLP,gdSLP1,gdSLP,iangSLP
c     WRITE(*,*) ''
c     ----------------------------------------------------

      gdELN=gELN(2,2)
c     WRITE(*,*) '<<calc3x3 (Sobel',
c    &          ' av,sd,sl)',gdELN,gdSLP,gdSDV

      END
c     =========================================================




c     ==================================================================
      SUBROUTINE getTOPO(rTOPO)

      INTEGER*2 iTOPO(10800)
      REAL pi,d2r
      CHARACTER*1 c00,c34,c39

      COMMON /CELL/cellLAT,cellLON
      COMMON /CONST/pi,d2r,c00,c34,c39

c     WRITE(*,*) '>>getTOPO: ',cellLAT,cellLON
c        rTOPO=9999.

c     These cell coords in ETOPO2 are needed
      iROW=INT((90.-cellLAT)*30.+1.)
      iCOL=INT((180.+cellLON)*30.+1.)

c     Get the value from the Direct Access ETOPO2 file
      READ(2,REC=iROW) (iTOPO(iW),iW=1,10800)
      rTOPO=REAL(iTOPO(iCOL))

c     WRITE(*,*) '<<getTOPO (Row/Col/rTOPO=',
c    &                   iROW,iCOL,rTOPO,')'

      END

c     ==================================================================




c     ==================================================================
      SUBROUTINE getHYPO(rLAT,rLON,rHYPO,sdHYPO,slHYPO)

      INTEGER*2 iTOPO(10800)
      REAL pi,d2r
      CHARACTER*1 c00,c34,c39
      REAL hLATcos

      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /NUL/rNUL,rnul9999,rnul00

      hLATcos=COS(rLAT*d2r)
      etopoCELLsze=2./60.

c     This routine does Hyperbolic interpolation
c        as described in Tetzlaff&Hedberg1989 for their module HSLOP

c     WRITE(*,*) '>>getHYPO: (rLAT,rLON=',rLAT,rLON,')'
         rHYPO=rnul9999
         sdHYPO=rNUL
         slHYPO=rNUL

c     Where does the Station lie in the 4 nodes surrounding ?
      dX=AMOD(rLAT*30.,1.)
      dY=AMOD(rLON*30.,1.)
c     (These are the ratio distances from the bottom left hand corner)
c     WRITE(*,*) 'Relative Locn to nodes: (dX,dY=',dX,dY,')'

c     These cell coords in ETOPO2 are needed
c     A1:
      iROWa1=INT((90.-rLAT)*30.+1.)
      iCOLa1=INT((180.+rLON)*30.+1.)
c     A2:
c     iROWa2=INT((90.-rLAT)*30.+1.)    not used
      iCOLa2=INT((180.+rLON)*30.+1.+.5)
c     A3:
      iROWa3=INT((90.-rLAT)*30.+1.+.5)
      iCOLa3=INT((180.+rLON)*30.+1.+.5)
c     A4:
c     iROWa4=INT((90.-rLAT)*30.+1.+.5)    not used
      iCOLa4=INT((180.+rLON)*30.+1.)
c     WRITE(*,*) '  a1      a2 ',iROWa1,iCOLa1,'    ',iROWa2,iCOLa2
c     WRITE(*,*) '      x      '
c     WRITE(*,*) '  a4      a3 ',iROWa4,iCOLa4,'    ',iROWa3,iCOLa3
c     WRITE(*,*) ''

c     Get the value from the Direct Access ETOPO2 file
c     A1,A2:
      READ(2,REC=iROWa1) (iTOPO(iW),iW=1,10800)
      rHYPOa1=REAL(iTOPO(iCOLa1))
      rHYPOa2=REAL(iTOPO(iCOLa2))
c     A3,A4:
      READ(2,REC=iROWa3) (iTOPO(iW),iW=1,10800)
      rHYPOa3=REAL(iTOPO(iCOLa3))
      rHYPOa4=REAL(iTOPO(iCOLa4))
c     WRITE(*,*) '  a1      a2 ',rHYPOa1,'    ',rHYPOa2
c     WRITE(*,*) '      x      '
c     WRITE(*,*) '  a4      a3 ',rHYPOa4,'    ',rHYPOa3
c     WRITE(*,*) ''

c     And what is the SDev of these TOPO values ?
      avHYPO=(rHYPOa1+rHYPOa2+rHYPOa3+rHYPOa4)/4.
      sdHYPO=(rHYPOa1-avHYPO)**2 +
     &       (rHYPOa2-avHYPO)**2 +
     &       (rHYPOa3-avHYPO)**2 +
     &       (rHYPOa4-avHYPO)**2 
      sdHYPO=SQRT(sdHYPO)

c     Do the Hyperbolic interpolation:
      rHYPO=rHYPOa1*dX*dY+rHYPOa2*(1-dX)*dY
     &     +rHYPOa3*dX*(1-dY)+rHYPOa4*(1-dx)*(1-dY)
c     (This was tested by CJJ 16Aug04 in 'tst_bathy_hyper.xls'
c      .. and verified ) 

c     Calculate slope on the 2x2 matrix ------------------
c        (really should use 3x3: do later CJJ Sep04)
c        The points are in order -1-1,-1+1,+1-1,+1+1
         aCOEF=(rHYPOa4+ rHYPOa1+ rHYPOa3+ rHYPOa2)/4.
         bCOEF=(-rHYPOa4+ rHYPOa1- rHYPOa3+ rHYPOa2)/4.
         cCOEF=(-rHYPOa4- rHYPOa1+ rHYPOa3+ rHYPOa2)/4.
         bCOEF=bCOEF*2.
         cCOEF=cCOEF*2.
c        bCOEF=yGRAD  cCOEF=xGRAD
c        WRITE(*,*) 'aCOEF,bCOEF,cCOEF',aCOEF,bCOEF,cCOEF
c        WRITE(*,*) hLATcos,etopoCELLsze,etopoCELLsze*60.*1862.
         slHYPO=SQRT(bCOEF**2 + (cCOEF/hLATcos)**2)
     &            /(etopoCELLsze*60.*1862.)
c        WRITE(*,*) SQRT(bCOEF**2 + (cCOEF/hLATcos)**2)
c    &            ,(etopoCELLsze*60.*1862.)
         IF (bCOEF /= 0.) THEN
            asHYPO=ATAN(-cCOEF/bCOEF)/d2r
         ELSE
            IF (cCOEF>0.) asHYPO=270.
            IF (cCOEF<0.) asHYPO=90.
         ENDIF
         IF (slHYPO.EQ.0.) asHYPO=0.
c        WRITE(*,*) 'slHYPO,asHYPO',slHYPO,asHYPO
c        (Based on UBC.ca course notes at 
c         'http://www.geog.ubc.ca/courses/klink/gis.notes/ncgia/u38.html'
c     ----------------------------------------------------

c     WRITE(*,*) '<<getHYPO (Row/Col(2)/rHYPO=',
c    &                    iROWa2,iCOLa2,rHYPO,')'

      END
c     ==================================================================




c     ==================================================================
      SUBROUTINE constructDA(iRECLbin)

      INTEGER*2 ETOPOb(10800),iC
      INTEGER*4 iDArec,iRECLbin
      INTEGER*1 TEST

      WRITE(*,*) '>>constructDA'

c     Get the ETOPO2 data from Binary and write to a Direct-Access file
c         with each record one circle of latitude

      OPEN(1,FILE='C:\dbSEABED\_db9\_Bathy\_Etopo2\ETOPO2.bin',
     &     FORM='UNFORMATTED')
      OPEN(2,FILE='C:\dbSEABED\_db9\_Bathy\_Etopo2\ETOPO2.dab',
     &        ACCESS='DIRECT',RECL=iRECLbin,FORM='UNFORMATTED')

      DO iR=1,5400
         READ(1,iostat=TEST) (ETOPOb(iC),iC=1,10800)

c        Echo example data to screen
         WRITE(*,*) (ETOPOb(iC),iC=1,10)
c        WRITE(*,801) (ETOPOb(iC),iC=1,10)
c801      FORMAT(10800I6)

c        Write current coordinate to screen
         rLAT=90.-iR/30.
         IF ((mod(rLAT,5.).EQ.0.0).AND.(MOD(iR,30).EQ.0.0)) THEN
            WRITE(*,815) 'Latitude=',rLAT
815         FORMAT(A9,i5)
cc          WRITE(*,815) 'Latitude=',rLAT,(iR-1)*10800+1
         ENDIF

c        Write to directACCESS file
         iDArec=iR
         WRITE(2,REC=iDArec) (ETOPOb(iC),iC=1,10800)
ccc      WRITE(*,801) (ETOPOb(iC),iC=1,10)

      ENDDO

      WRITE(*,*) '                 '
      WRITE(*,*) 'Direct Access File constructed '
      WRITE(*,*) 'with total of ',iDArec,' records'

      CLOSE(1)
      CLOSE(2)
cc    CLOSE(3)

      END
c     ==================================================================




c     =========================================================
      FUNCTION rMAXMIN(mxmnS,a,b,rMXMNnul)

      CHARACTER*3 mxmnS

c     rMXMNnul may differ from call to call

      IF ((a.EQ.rMXMNnul).AND.(b.EQ.rMXMNnul)) THEN
         rMAXMIN=rMXMNnul
         WRITE(*,*) 'warning rMAXMIN=rMXMNnul'
      ELSEIF (a.EQ.rMXMNnul) THEN
         rMAXMIN=b
      ELSEIF (b.EQ.rMXMNnul) THEN
         rMAXMIN=a
      ELSEIF (mxmnS.EQ.'max') THEN
         IF (a >= b) rMAXMIN=a
         IF (b >= a) rMAXMIN=b
      ELSEIF (mxmnS.EQ.'min') THEN
         IF (a <= b) rMAXMIN=a
         IF (b <= a) rMAXMIN=b
      ENDIF

c     WRITE(*,*) '<<MaxMin',a,b,rMXMNnul

      END
c     =========================================================




c     =========================================================
      FUNCTION str2num(sNUM)

      INTEGER iNUM
      REAL*4 str2num
      CHARACTER*16 sNUM
      REAL*4 pi,d2r
      CHARACTER*1 c00,c34,c39

      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /NUL/rNUL,rnul9999,rnul00

c     WRITE(*,*) '>>str2num: \',sNUM(1:LEN_TRIM(sNUM)),'\'
      rNUM0=0
      str2num=rNUL

c     Deal with non-numerics ------------------
      IF ((sNUM.EQ.c00).OR.(sNUM.EQ.'-')) THEN
         rNUM=rNUL
         GOTO 555
      ENDIF
c     -----------------------------------------

      DO iNM=1,LEN_TRIM(sNUM)
         IF (sNUM(iNM:iNM).EQ.'.') goto 772
         IF (sNUM(iNM:iNM).EQ.'+') goto 772
         IF (sNUM(iNM:iNM).EQ.'-') goto 772

         iNUM=ICHAR(sNUM(iNM:iNM))-48
         rNUM0=rNUM0*10.+REAL(iNUM)
c        WRITE(*,*) '&&&',iNM,iNUM,rNUM0
772      CONTINUE
      ENDDO

      iDECpt=INDEX(sNUM,'.')
      IF (iDECpt>0.0) THEN
         iSHIFT=LEN_TRIM(sNUM)-iDECpt
         rNUM=rNUM0/10.**(iSHIFT)
      ELSE
c        Remains unchanged
         rNUM=rNUM0
      ENDIF

c     WRITE(*,*) sNUM(1:1),'%%%%%',(sNUM(1:1).EQ.'-') 
      IF (sNUM(1:1).EQ.'-') rNUM=-rNUM

555   str2num=rNUM

c     WRITE(*,*) '### \',sNUM(1:LEN_TRIM(sNUM)),'\',
c    &           rNUM0,iDECpt,iSHIFT,rNUM
c     WRITE(*,*) '<<str2num: \',sNUM(1:LEN_TRIM(sNUM)),'\',rNUM

      END
c     =========================================================




c     =========================================================
      FUNCTION num2str(rNUM)

      CHARACTER*16 numS,num2str
      CHARACTER*16 num2str0
      CHARACTER*12 fnameS

c     WRITE(*,*) '>>num2str: |',rNUM,'|'
      fnameS='_num2str.txt'
      WRITE(fnameS,'(f16.6)') rNUM
      READ(fnameS,'(a16)') numS

c     Fill blanks
      DO iC=1,16
         IF (numS(iC:iC).EQ.' ') THEN
            num2str0=num2str0(1:iC-1)//'0'
         ELSE
            num2str0=num2str0(1:iC-1)//numS(iC:iC)
         ENDIF
      ENDDO
      num2str=num2str0

c     WRITE(*,*) '<<num2str: |',num2str0,'|'
c     Output format is: "000000000.000000"

      END
c     =========================================================




c     =========================================================
      SUBROUTINE breakUpLINE(vS,lineS)

      CHARACTER*512 lineS
      CHARACTER*64 vS(24)
      CHARACTER*1 charB
      INTEGER v

c     WRITE(*,*) '>>: breakUPline'

      iSTARTv=1
      charB=','
      DO v=1,32
         iCOMMApos=INDEX(lineS(iSTARTv:LEN(lineS)),charB)
c        WRITE(*,*) '***',v,iSTARTv,iCOMMApos,LEN_TRIM(lineS)

         IF (iCOMMApos.EQ.0) GOTO 777

         vS(v)=lineS(1:iCOMMApos-1)
c        WRITE(*,*) vS(v)
         vS(v)=vS(v)(LTRIM(vS(v)):LEN_TRIM(vS(v)))
c        WRITE(*,*) 'bkUpln:',v,'|',vS(v)(1:LEN_TRIM(vS(v))),'|'

         lineS=lineS(iCOMMApos+1:LEN_TRIM(lineS))
c        WRITE(*,*) lineS(1:LEN_TRIM(lineS))

      ENDDO
777   vS(24)=lineS(iCOMMApos+1:LEN_TRIM(lineS))
c     WRITE(*,*) vS(24)
      vS(24)=vS(24)(LTRIM(vS(24)):LEN_TRIM(vS(24)))
c     WRITE(*,*) 'bkUpln:',v,'|',vS(24)(1:LEN_TRIM(vS(24))),'|'

c     WRITE(*,*) '<<: breakUPline'
c     WRITE(*,*) 'vS: ',(vS(v)(1:LEN_TRIM(vS(v))),v=1,32)

      END
c     =========================================================




c     =========================================================
      FUNCTION LTRIM(strS)
c     Left Trim

      CHARACTER*512 strS

c    WRITE(*,*) '>>LTRIM: |',strS,'|'

      DO iC=1,LEN_TRIM(strS)
         IF (strS(iC:iC).EQ.' ') THEN
c           Do nothing (go on)
         ELSE
            iC0=iC
c           WRITE(*,*) 'non-blank found at:',iC0
            GOTO 540
         ENDIF
      ENDDO

540   LTRIM=iC0
c     WRITE(*,*) '<<LTRIM: ',iC0,LEN_TRIM(strS)
c     WRITE(*,*) ''

      END
c     =========================================================




c     =========================================================
      FUNCTION midS(strS,startN,stopN)

      CHARACTER strS*512,midS*126
      INTEGER startN,stopN

c     WRITE(*,*) '>>midS: /',strS,'/',startN,stopN

      midS=strS(startN:startN+stopN-1)

c     WRITE(*,*) '<<midS: /',strS(startN:startN+stopN-1),'/'
c     CALL SLEEP(3)

      END
c     =========================================================




c     =========================================================
      FUNCTION ran0(idum)

      INTEGER idum,IA,IM,IQ,IR,MASK
      REAL ran0,AM
      PARAMETER (IA=16807,IM=2147483647,AM=1./IM,
     &           IQ=127773,IR=2836,MASK=123459876)
      INTEGER k

c      "Minimal" random number generator of Park and Miller. 
c           Returns a uniform random deviate
c           between 0.0 and 1.0. Set or reset idum to any integer value 
c            (except the unlikely value MASK)
c           to initialize the sequence; idum must not be altered between 
c           calls for successive deviates in a sequence.

      idum=ieor(idum,MASK) 
c     XORing with MASK allows use of zero and 
c        other simple bit patterns for idum. 
      k=idum/IQ
      idum=IA*(idum-k*IQ)-IR*k 
c     Compute idum=mod(IA*idum,IM) without overflows by Schrage's method. 
      if (idum.lt.0) idum=idum+IM
      ran0=AM*idum 
c     Convert idum to a floating result.
      idum=ieor(idum,MASK) 
c     Unmask before return.

      RETURN

      END
c     =========================================================




c     =========================================================
      FUNCTION decPT(rNUM,iDCpt)

      REAL rNUM

c      Chenges the precision (deg points) of a REAL 

         decPT=NINT(rNUM*(10.**iDCpt))/(10.**iDCpt)

      RETURN

      END
c     =========================================================




c     =========================================================
      SUBROUTINE sortN(N0,RA)

      INTEGER N0
      REAL RA(24)

c     WRITE(*,*) '>>sortN: ',N0
c     WRITE(*,*) 'RA:',(RA(iNw),iNw=1,N0)

c     Pad to make even N
      IF (N0/2 /= INT(REAL(N0)/2.0+0.5)) THEN
c        WRITE(*,*) 'Pad'
         N=N0+1
         RA(N0+1)=-99.
c        This is probably an unlikely number
c         (will trim the bottom - minimum - off array on RETURN with arraysize=N
      ELSE
         N=N0
      ENDIF

      L=N/2+1
      IR=N

10    CONTINUE
         IF (L>1) THEN
            L=L-1
            RRA=RA(L)
         ELSE
            RRA=RA(IR)
            RA(IR)=RA(1)
            IR=IR-1
            IF (IR.EQ.1) THEN
               RA(1)=RRA
               GOTO 30
            ENDIF
         ENDIF
         I=L
         J=L+L
20       IF (J <= IR) THEN
            IF (J<IR) THEN
               IF (RA(J)<RA(J+1)) J=J+1
            ENDIF
            IF (RRA<RA(J)) THEN
               RA(I)=RA(J)
               I=J
               J=J+J
            ELSE
               J=IR+1
            ENDIF
         GOTO 20
         ENDIF
         RA(I)=RRA
      GOTO 10

30    CONTINUE
c     WRITE(*,*) '<<sortN:',N0
c     WRITE(*,*) 'RA:',(RA(iNw),iNw=1,N0)

      END
c     =========================================================




c     =========================================================
      SUBROUTINE coastalCLASSN

      INTEGER*2 iTOPO(10800)
      INTEGER*4 gELN(3,3)
      INTEGER nCOLS,nROWS

      REAL latMN,lonMN
      REAL gPRX(3,3)
      REAL dgCELLsz,kmSRCHradMX
      REAL pi,d2r
      REAL xSB(3,3),ySB(3,3)

      CHARACTER*1 c00,c34,c39
      CHARACTER workDIRS*25,projectS*3
      CHARACTER*8 cCST
      CHARACTER*3 typSRCHrad,adjstSRCHrad
      CHARACTER fnmSUFFXs*3

      COMMON /BOX/lonMN,latMN,BOXtop,boxBOT,nCOLS,nROWS
      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /NUL/rNUL,rnul9999,rnul00
      COMMON /PROJ/projectS,workDIRS
      COMMON /RADS/dgCELLsz,kmSRCHradMX,adjstSRCHrad,typSRCHrad
      COMMON /SUFFIX/fnmSUFFXs

c     Sobel filter parameters ---
      DATA xSB/-1.,+0.,+1.,
     &         -2.,+0.,+2.,
     &         -1.,+0.,+1./
      DATA ySB/-1.,-2.,-1.,
     &         +0.,+0.,+0.,
     &         +1.,+2.,+1./

      WRITE(*,*) '>>coastalCLASSN: '

c     File structures -------------------------------
c     A spatial analyst grid:
      OPEN(20,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'c.asc')
c     Coastal Relief grid (m)
      WRITE(20,280) 'ncols          ',nCOLS
      WRITE(20,280) 'nrows          ',nROWS
      WRITE(20,281) 'xllcorner    ',lonMN
      WRITE(20,281) 'yllcorner    ',latMN
      WRITE(20,281) 'cellsize     ',dgCELLsz
      WRITE(20,283) 'NODATA_value ',rnul9999
280   FORMAT(a13,i5)
281   FORMAT(a13,f10.6)
283   FORMAT(a13,i3)

c     Append the data to the main input data file
      OPEN(4,FILE=workDIRS//projectS//'_WWD.txt',
     & ACCESS='APPEND')

c     Will also need a land mask (prx=0)
c     The working proximity file is ASCII direct access 
      OPEN(1,FILE=workDIRS//'_'//projectS//'_prx.tmp',
     &       ACCESS='DIRECT',RECL=6,FORM='FORMATTED')

c     A text point listing (the most useful for plotting):
      OPEN(21,FILE=workDIRS//projectS//'_'//fnmSUFFXs//'c.txt')
      WRITE(21,284) 'Lat,Lon,gdMAX,avELN,sdELN,gdSLP'
284   FORMAT(a)
c     -----------------------------------------------

      DO iCyC=nROWS,1,-1
         cellLAT=latMN+dgCELLsz*(REAL(iCyC-1)+.5)
         DO iCxC=1,nCOLS
            cellLON=lonMN+dgCELLsz*(REAL(iCxC-1)+.5)

            cosLAT=COS(cellLAT*d2r)

            gdMAX=rnul9999
            gdSLP=rNUL
            jMIDlndYN=0
c           jPERseaYN=0
            jPERprxYN=0
            rmaxCST=rnul9999
            DO iR=1,3
            DO iC=1,3
               gELN(iR,iC)=-99
            ENDDO
            ENDDO
                     jS=-99
                     jM=-99
                     jG=-99
                     rGz=-99
                     cCST='-'


c              Get the 3x3 set for Elevations --------------------
c              These cell coords in ETOPO2 are needed
c              WRITE(*,*) 'iCxC,iCyC:',iCxC,iCyC
c              WRITE(*,*) 'cellLAT/LON',cellLAT,cellLON
               DO iR=1,3
                  iROW=INT((90.-cellLAT)*30.+1.)+(iR-2)
c                 WRITE(*,*) 'iROW',iROW,iR
c                 Get the values from the Direct Access ETOPO2 file
                  READ(2,REC=iROW) (iTOPO(iW),iW=1,10800)
                  DO iC=1,3
c                    WRITE(*,*) 'iROW,iCOL',iROW,iCOL,iR,iC
                     iCOL=INT((180.+cellLON)*30.+1.)+(iC-2)
                     gELN(iR,iC)=iTOPO(iCOL)
                  ENDDO
               ENDDO
c              WRITE(*,*) 'gELN:',((gELN(iR,iC),iC=1,3),iR=1,3)
c              WRITE(*,*) INT((90.-cellLAT)*30.+1.),
c    &              INT((180.+cellLON)*30.+1.)
c              ---------------------------------------------------

c              Get the 3x3 set for Proximity ---------------------
               DO iR=1,3
                  DO iC=1,3
                     iprxNUM=(nROWS-(iCyC+(iR-2)))
     &                        *nCOLS+(iCxC+(iC-2))
                     IF ((iCxC+(iC-2)>nCOLS).OR.
     &                    (iCxC+(iC-2)<1)) THEN
                        gPRX(iR,iC)=-99.
c                       WRITE(*,*) 'Out of bounds',iprxNUM
                     ELSEIF ((iCyC+(iR-2)>nROWS).OR.
     &                    (iCyC+(iR-2)<1)) THEN
                        gPRX(iR,iC)=-99.
c                       WRITE(*,*) 'Out of bounds',iprxNUM
                     ELSE
                        READ(1,'(f6.3)',REC=iprxNUM) gPRX(iR,iC)
                     ENDIF
                  ENDDO
               ENDDO
c              WRITE(*,*) 'gPRX:',((gPRX(iR,iC),iC=1,3),iR=1,3)
c              ---------------------------------------------------

c              CONDITIONS: -------------------
c              A. cell is in land
               if (INT(gPRX(2,2)/0.02+0.5).EQ.0) jMIDlndYN=1

c              Test in the 3x3 window ----------------------
               DO iR=1,3
                  DO iC=1,3

c                 B. a marine cell is in ortho-periphery 
c                    (and not in middle)
c                    Commented out CJJ Sep 2007
cc                   IF ((iR /= 2).AND.(iC /= 2)) THEN
cc                      IF (gELN(iR,iC)<0) jPERseaYN=1
cc                   ENDIF

c                 C. cell with prx=~minimum is in ortho-periphery
c                    (Dont test if is already one known)
                      IF (((iR.EQ.2).OR.(iC.EQ.2)).AND.
     &                   ((iR /= 2).OR.(iC /= 2))) THEN
c                        [ this is ortho periphery ]
c                        [IF ((iR /= 2).AND.(iC /= 2)) THEN
c                         this is all periphery ]
                        IF (INT(gPRX(iR,iC)/0.02+0.5).EQ.1) 
     &                       jPERprxYN=1
                     ENDIF
                  ENDDO
               ENDDO
c              ---------------------------------------------

c              Calculate coastal classification=======================
c              WRITE(*,*) 'Value the cell'
               IF ((jPERprxYN.EQ.1).AND.(jMIDlndYN.EQ.1)) THEN
c                 Classifier=max elevation  >= -1
                  DO iR=1,3
                     DO iC=1,3
                        rmaxCST=rMAXMIN('max',rmaxCST,
     &                      REAL(gELN(iR,iC)),rnul9999)
                     ENDDO
                  ENDDO
                  gdMAX=rmaxCST

c                 And other statistics 
c                 Calculate sum
                  sumELN=0.
                  sumELN2=0.
                  DO iC=1,3
                     DO iR=1,3
                        sumELN=sumELN+gELN(iR,iC)
                     ENDDO
                  ENDDO
                  avELN=sumELN/9.
c                 Calculate SD
                  DO iC=1,3
                     DO iR=1,3
                        sumELN2=sumELN2+(gELN(iR,iC)-avELN)**2
                     ENDDO
                  ENDDO
                  sdELN=SQRT(sumELN2/9.)
                  WRITE(*,*) 'Basic stats: (avELN,sdELN=)',
     &                               avELN,sdELN

c                 Calculate Sobel X,Y slopes 
                  xSLP=gELN(1,1)*xSB(1,1)   +gELN(1,3)*xSB(1,3)
     &                +gELN(2,1)*xSB(2,1)   +gELN(2,3)*xSB(2,3)
     &                +gELN(3,1)*xSB(3,1)   +gELN(3,3)*xSB(3,3)
                  xSLP=xSLP/8.

                  ySLP=gELN(1,1)*ySB(1,1)+gELN(1,2)*ySB(1,2)
     &                +gELN(1,3)*ySB(1,3)
     &                +gELN(3,1)*ySB(3,1)+gELN(3,2)*ySB(3,2)
     &                +gELN(3,3)*ySB(3,3)
                  ySLP=ySLP/8.
                  gdSLP1=SQRT(xSLP**2+ySLP**2)

c                 And normalize to grid size of 2min X&Y
                  gdSLP2=SQRT((2./60.)**2+(2./60.*cosLAT)**2) 
     &                    *1832.*60.
                  gdSLP=gdSLP1/gdSLP2
                  WRITE(*,*) 'Sobel outputs: (xSLP,ySLP,gdSLP=)',
     &                                  xSLP,ySLP,gdSLP1,gdSLP

               ELSE
c                 Not a coastal cell
                  gdMAX=9999.
               ENDIF
c                 =====================================================

c              Write some point data outputs
               IF (gdMAX /= 9999.) THEN
                  WRITE(*,*) ''
c                 WRITE(*,*) 'gELN:',((gELN(iR,iC),iC=1,3),iR=1,3)
c                 WRITE(*,*) 'iCxC/yC,cellL/L,gdMAX:',
c    &                      iCxC,iCyC,cellLAT,cellLON,gdMAX,gdSLP
c                 WRITE(21,185) cellLAT,',',cellLON,',',
c    &                  gdMAX,',',avELN,',',sdELN,',',gdSLP
c185               FORMAT(2(f9.5,a1),3(f6.0,a1),f6.4)

c                 Make an actual data file:
                  IF (gdMAX <= 3) THEN
                     jS=70
                     jM=20
                     jG=10
                     rGz=3.0
                     cCST='LO_COAST'
                  ELSE
                     jS=75
                     jM=5
                     jG=20
                     rGz=1.0
                     cCST='HI_COAST'
                  ENDIF
                  IF (gdMAX>2.) THEN
                     jRk=INT(1./(1.+3./REAL(gdMAX))**2 *100.)
                  ELSE
                     jRk=0
                  ENDIF

c                 Append the data to the *_WWD.txt file in "../_Interp"
c                 "Latitude","Longitude","WaterDepth",
c                 "SampleTop","SampleBase","SiteName",
c                 "DataSetKey","SiteKey","SampleKey"
c                 "Sampler","DataTypes"
c                 "Gravel","Sand","Mud","Clay","Grainsize","Sorting"
c                 "SeabedCls","ClsMshp","ShepardCode",
c                 "RockMshp","WeedMshp",
c                 "Carbonate","MunslCode","OrgCarbon"
c                 "LgShearStr","Porosity","PWaveVel"
c                 "Roughness","LgCritShrStrs","SamplePhase"
cc                WRITE(*,*) cellLAT,cellLON,INT(-avELN+0.5),
cc   &                   jG,jS,jM,rGz,
cc   &                   cCST,jRk
                  WRITE(4,394) cellLAT,cellLON,INT(-avELN+0.5),
     &                   jG,jS,jM,rGz,
     &                   cCST,jRk
394               FORMAT(2(f9.5,','),(i6,','),
     &                   '0.0,0.0,"CstClsn",',
     &                   '-99,-99,-99,',
     &                   '"Geomorph",GGGxGxxxGGxxxxxxxxxx,',
     &                   3(i3,','),'-99,',(f4.1,','),'-99,',
     &                   ('"',a8,'",'),('-99,"-",'),
     &                   (i3,','),'-99,',
     &                   '-99,"-",-99,',
     &                   '-99,-99,-99,',
     &                   '"-",-99,"-"')
                  WRITE(*,*) 'Coast type: |',cCST,'|'

               ENDIF
c              And the grid ...
               WRITE(20,'(f6.0,a1)') gdMAX,' '

         ENDDO
c        Grid rows
      ENDDO
c     Grid columns

      CLOSE(1)
      CLOSE(4)

      CLOSE(20)
      CLOSE(21)

      WRITE(*,*) '<<coastalCLASSN: '
      CALL SLEEP(3)

      END
c     =========================================================




c     =========================================================
      SUBROUTINE setSRCH(iCx,iCy,kmSRCHrad)

      INTEGER nCOLS,nROWS

      REAL pi,d2r
      REAL gdELN,gdSLP,gdSDV
      REAL kmCELLsz,dgCELLsz,kmSRCHrad,kmSRCHradMX
      REAL cosLAT

      CHARACTER*3 typSRCHrad,adjstSRCHrad
      CHARACTER*1 c00,c34,c39

      COMMON /BOX/lonMN,latMN,BOXtop,boxBOT,nCOLS,nROWS
      COMMON /CELL/cellLAT,cellLON
      COMMON /CONST/pi,d2r,c00,c34,c39
      COMMON /ELN/gdELN,gdSLP,gdSDV,gdMAX
      COMMON /RADS/dgCELLsz,kmSRCHradMX,adjstSRCHrad,typSRCHrad

c     WRITE(*,*) '>>setSRCH (iCx/y,adj/typSRCHrad=):',
c    &                iCx,iCy,' ',adjstSRCHrad,' ',typSRCHrad

      cosLAT=COS(cellLAT*d2r)
c     WRITE(*,*) 'cosLAT,d2r',cosLAT,d2r
      kmCELLsz=dgCELLsz*(60.*1.832*cosLAT)

c     What type of search radius ? ---------------------------
      IF ((adjstSRCHrad.EQ.'FXD').OR.(adjstSRCHrad.EQ.'FIX')) THEN
         kmSRCHrad=kmSRCHradMX
c        WRITE(*,*) 'Fixed kmSRCHrad:',
c    &              kmSRCHradMX,kmSRCHrad

      ELSEIF (adjstSRCHrad.EQ.'BCV') THEN
         gNORM=1.0
         kmSRCHrad=AMIN1(kmSRCHradMX,
     &           kmSRCHradMX/(1.+gNORM*(gdSDV/ABS(gdELN))) )
c        WRITE(*,*) 'Adaptive kmSRCHrad by BCV:',
c    &              kmSRCHradMX,gdSDV/gdELN,kmSRCHrad

      ELSEIF (adjstSRCHrad.EQ.'BTY') THEN
c        Decrease by BTY (gNORM/BTY,where that is less than 1; 
c          gNORM m is the elevation at which max SRCH radius is achieved)
         gNORM=-20.0
         IF (gdELN<0) THEN
            kmSRCHrad=AMIN1(kmSRCHradMX,kmSRCHradMX*ABS(gdELN/gNORM))
         ELSE
            kmSRCHrad=kmCELLsz
         ENDIF
c        WRITE(*,*) 'Adaptive kmSRCHrad by BTY/20:',
c    &              kmSRCHradMX,gdELN/gNORM,kmSRCHrad

      ELSEIF (adjstSRCHrad.EQ.'PRX') THEN

c        Get the PRX value (in deg in the PRX grid)
         iprxNUM=(nROWS-iCy)*nCOLS+iCx
c        (Sequential index to proximity grid in DirAccs file)
         READ(1,'(f6.3)',REC=iprxNUM) cellPRX
         cellPRXkm=cellPRX*(60.*1.832*cosLAT)

c        Then pro-rata search radius by proximity
c        (Option 1)------
c        Simply the proximity
c        kmSRCHrad=AMIN1(kmSRCHradMX,cellPRXkm)

c        (Option 2)------
c        Decrease by PRX (gNORM/PRX,where that is less than 1; 
c          gNORM km is the proximity at which srch radius becomes the max;
c          it cannot be larger than the maxSRCHradMX)
         gNORM=AMIN1(10.,kmSRCHradMX)
c        kmSRCHradMX is the outboard wider search radius
c        The search radius shrinks linearly inside distance gNORM
         kmSRCHrad=AMIN1(kmSRCHradMX,kmSRCHradMX*(cellPRXkm/gNORM))
c        But later the minimum of 1 cell search radius is reinstated

c        WRITE(*,*) 'Adaptive kmSRCHrad by PRX:(kSr1/cPXk/Sr)',
c   &              kmSRCHradMX,cellPRXkm,kmSRCHrad
      ELSE
         WRITE(*,*) 'No proper searchDIAM type specified |',
     &              adjstSRCHrad,'|'
         CALL SLEEP(3)
         STOP
      ENDIF

c     A maximum search radius is set
      IF (kmSRCHrad>kmSRCHradMX) kmSRCHrad=kmSRCHradMX

c     But the minimum search radius is to be 1 cell
      kmSRCHrad=AMAX1(kmSRCHrad,dgCELLsz*(60.*1.832*cosLAT))
c     The 1 cell radius limit is interim: CJJ Dec04
c     WRITE(*,*) 'kmSRCHrad(rad,typ,radMX=):',
c    &                     kmSRCHrad,typSRCHrad,kmSRCHradMX
c     --------------------------------------------------------

      END
c     =========================================================




c     =========================================================
c     =========================================================
c     =========================================================
      SUBROUTINE CALIBpoints(inbox,iD,rLAT,rLON,siteWD,subTOP,subBOT,
     &        rPARAM,ePARAM,ptELN,ptSDV,ptSLP,metaDS)

      REAL ptELN,ptSLP,ptSDV

      CHARACTER calibrateS*1
      CHARACTER*1 metaDS

      COMMON /CALIB/calibrateS

c        Should the data be held back for calibration ? -----
c        (Do this with a random number for 10%)

c        2 possible methods:

c           Option 1:
c           IF (calibrateS.EQ.'y') ichuzPNT=NINT(RAN0(idumRAN))*10
c              (and set ready for next RAN0)
c              idumRAN=ichuzPNT

c           Option 2:
            IF (calibrateS.EQ.'y') ichuzPNT=MOD(iD,10)
c           (Chooses every 10th)
c           WRITE(*,*) 'Extract calibration point ?',
c    &         iD,ichuzPNT,calibrateS

         IF ((ichuzPNT.EQ.1).AND.(inBOX.EQ.1)) then
c           WRITE(*,*) 'Calibration point (10%):',
c    &         iD,ichuzPNT," ",calibrateS

            WRITE(10,239) rLAT,rLON,siteWD,subTOP,subBOT,
     &                   rPARAM,ePARAM,
     &                   ptELN,ptSDV,ptSLP,metaDS
239         FORMAT(2(f10.5,','),f7.1,',',2(f6.2,','),
     &                   2(f6.2,','),
     &                   f7.0,',',f7.2,',',f7.2,',',a1)
            iCALIB=iCALIB+1
c           Unflag from working data set
            inBOX=-1
         ENDIF
c     -------------------------------------------------------

      END
c     =========================================================





c     =========================================================
